# (C) Copyright 2005-2020 Enthought, Inc., Austin, TX
# All rights reserved.
#
# This software is provided without warranty under the terms of the BSD
# license included in LICENSE.txt and may be redistributed only under
# the conditions described in the aforementioned license. The license
# is also available online at http://www.enthought.com/licenses/BSD.txt
#
# Thanks for using Enthought open source!

""" A panel sized by a sizer. """


import wx


class SizedPanel(wx.Panel):
    """ A panel sized by a sizer. """

    def __init__(self, parent, wxid, sizer, **kw):
        """ Creates a new sized panel. """

        # Base-class constructor.
        wx.Panel.__init__(self, parent, wxid, **kw)

        # Set up the panel's sizer.
        self.SetSizer(sizer)
        self.SetAutoLayout(True)

        # A quick reference to our sizer (at least quicker than using
        # 'self.GetSizer()' ;^).
        self.sizer = sizer

        return

    # ------------------------------------------------------------------------
    # 'SizedPanel' interface.
    # ------------------------------------------------------------------------

    def Fit(self):
        """ Resizes the panel to match the sizer's minimal size. """

        self.sizer.Fit(self)

    def Layout(self):
        """ Lays out the sizer without changing the panel geometry. """

        self.sizer.Layout()

        return



# (C) Copyright 2005-2020 Enthought, Inc., Austin, TX
# All rights reserved.
#
# This software is provided without warranty under the terms of the BSD
# license included in LICENSE.txt and may be redistributed only under
# the conditions described in the aforementioned license. The license
# is also available online at http://www.enthought.com/licenses/BSD.txt
#
# Thanks for using Enthought open source!


""" Defines the IDockUIProvider interface which objects which support being
    dragged and dropped into a DockWindow must implement.
"""

# -------------------------------------------------------------------------------
#  'IDockUIProvider' class:
# -------------------------------------------------------------------------------


class IDockUIProvider(object):

    # ---------------------------------------------------------------------------
    #  Returns a Traits UI which a DockWindow can imbed:
    # ---------------------------------------------------------------------------

    def get_dockable_ui(self, parent):
        """ Returns a Traits UI which a DockWindow can imbed.
        """
        return self.edit_traits(
            parent=parent, kind="subpanel", scrollable=True
        )



# (C) Copyright 2005-2020 Enthought, Inc., Austin, TX
# All rights reserved.
#
# This software is provided without warranty under the terms of the BSD
# license included in LICENSE.txt and may be redistributed only under
# the conditions described in the aforementioned license. The license
# is also available online at http://www.enthought.com/licenses/BSD.txt
#
# Thanks for using Enthought open source!
""" The default perspective menu for a workbench window. """


from pyface.action.api import Group, MenuManager
from traits.api import Instance, List, on_trait_change


from .delete_user_perspective_action import DeleteUserPerspectiveAction
from .new_user_perspective_action import NewUserPerspectiveAction
from .rename_user_perspective_action import RenameUserPerspectiveAction
from .reset_all_perspectives_action import ResetAllPerspectivesAction
from .reset_active_perspective_action import ResetActivePerspectiveAction
from .save_as_user_perspective_action import SaveAsUserPerspectiveAction
from .set_active_perspective_action import SetActivePerspectiveAction


class PerspectiveMenuManager(MenuManager):
    """ The default perspective menu for a workbench window. """

    # 'ActionManager' interface --------------------------------------------

    # All of the groups in the manager.
    groups = List(Group)

    # The manager's unique identifier.
    id = "PerspectivesMenu"

    # 'MenuManager' interface ---------------------------------------------#

    # The menu manager's name.
    name = "Perspectives"

    # 'PerspectiveMenuManager' interface -----------------------------------

    # The workbench window that the manager is part of.
    window = Instance("pyface.workbench.api.WorkbenchWindow")

    # ------------------------------------------------------------------------
    # 'ActionManager' interface.
    # ------------------------------------------------------------------------

    def _groups_default(self):
        """ Trait initializer. """

        groups = [
            # Create a group containing the actions that switch to specific
            # perspectives.
            self._create_perspective_group(self.window),
            # Create a group containing the user perspective create/save/rename
            # /delete actions.
            self._create_user_perspective_group(self.window),
            # Create a group containing the reset actions.
            self._create_reset_perspective_group(self.window),
        ]

        return groups

    # ------------------------------------------------------------------------
    # 'PerspectiveMenuManager' interface.
    # ------------------------------------------------------------------------

    @on_trait_change("window.perspectives")
    @on_trait_change("window.perspectives_items")
    def rebuild(self):
        """ Rebuild the menu.

        This is called when user perspectives have been added or removed.

        """

        # Clear out the old menu. This gives any actions that have trait
        # listeners (i.e. the rename and delete actions!) a chance to unhook
        # them.
        self.destroy()

        # Resetting the trait allows the initializer to run again (which will
        # happen just as soon as we fire the 'changed' event).
        self.reset_traits(["groups"])

        # Let the associated menu know that we have changed.
        self.changed = True

        return

    # ------------------------------------------------------------------------
    # Private interface.
    # ------------------------------------------------------------------------

    def _create_perspective_group(self, window):
        """ Create the actions that switch to specific perspectives. """

        # fixme: Not sure if alphabetic sorting is appropriate in all cases,
        # but it will do for now!
        perspectives = window.perspectives[:]
        perspectives.sort(key=lambda x: x.name)

        # For each perspective, create an action that sets the active
        # perspective to it.
        group = Group()
        for perspective in perspectives:
            group.append(
                SetActivePerspectiveAction(
                    perspective=perspective, window=window
                )
            )

        return group

    def _create_user_perspective_group(self, window):
        """ Create the user perspective create/save/rename/delete actions. """

        group = Group(
            NewUserPerspectiveAction(window=window),
            SaveAsUserPerspectiveAction(window=window),
            RenameUserPerspectiveAction(window=window),
            DeleteUserPerspectiveAction(window=window),
        )

        return group

    def _create_reset_perspective_group(self, window):
        """ Create the reset perspective actions. """

        group = Group(
            ResetActivePerspectiveAction(window=window),
            ResetAllPerspectivesAction(window=window),
        )

        return group



# (C) Copyright 2005-2020 Enthought, Inc., Austin, TX
# All rights reserved.
#
# This software is provided without warranty under the terms of the BSD
# license included in LICENSE.txt and may be redistributed only under
# the conditions described in the aforementioned license. The license
# is also available online at http://www.enthought.com/licenses/BSD.txt
#
# Thanks for using Enthought open source!

""" A simple progress bar intended to run in the UI thread """

import time

from pyface.qt import QtGui, QtCore

from traits.api import Bool, Instance, Int, Str, provides

from pyface.i_progress_dialog import IProgressDialog, MProgressDialog
from .window import Window


@provides(IProgressDialog)
class ProgressDialog(MProgressDialog, Window):
    """ A simple progress dialog window which allows itself to be updated

    """

    # FIXME: buttons are not set up correctly yet

    #: The progress bar widget
    progress_bar = Instance(QtGui.QProgressBar)

    #: The window title
    title = Str()

    #: The text message to display in the dialog
    message = Str()

    #: The minimum value of the progress range
    min = Int()

    #: The minimum value of the progress range
    max = Int()

    #: The margin around the progress bar
    margin = Int(5)

    #: Whether or not the progress dialog can be cancelled
    can_cancel = Bool(False)

    # The IProgressDialog interface doesn't declare this, but since this is a
    # feature of the QT backend ProgressDialog that doesn't appear in WX, we
    # offer an option to disable it.
    can_ok = Bool(False)

    #: Whether or not to show the time taken (not implemented in Qt)
    show_time = Bool(False)

    #: Whether or not to show the percent completed
    show_percent = Bool(False)

    #: The size of the dialog
    dialog_size = Instance(QtCore.QRect)

    #: Label for the 'cancel' button
    cancel_button_label = Str("Cancel")

    #: Whether or not the dialog was cancelled by the user
    _user_cancelled = Bool(False)

    #: The widget showing the message text
    _message_control = Instance(QtGui.QLabel)

    #: The widget showing the time elapsed
    _elapsed_control = Instance(QtGui.QLabel)

    #: The widget showing the estimated time to completion
    _estimated_control = Instance(QtGui.QLabel)

    #: The widget showing the estimated time remaining
    _remaining_control = Instance(QtGui.QLabel)

    # -------------------------------------------------------------------------
    # IWindow Interface
    # -------------------------------------------------------------------------

    def open(self):
        """ Opens the window. """
        super(ProgressDialog, self).open()
        self._start_time = time.time()

    def close(self):
        """ Closes the window. """
        self.progress_bar.destroy()
        self.progress_bar = None

        super(ProgressDialog, self).close()

    # -------------------------------------------------------------------------
    # IProgressDialog Interface
    # -------------------------------------------------------------------------

    def update(self, value):
        """ Update the progress bar to the desired value

        If the value is >= the maximum and the progress bar is not contained
        in another panel the parent window will be closed.

        Parameters
        ----------
        value :
            The progress value to set.
        """

        if self.progress_bar is None:
            return None, None

        if self.max > 0:
            self.progress_bar.setValue(value)

            if self.max != self.min:
                percent = (float(value) - self.min) / (self.max - self.min)
            else:
                percent = 1.0

            if self.show_time and (percent != 0):
                current_time = time.time()
                elapsed = current_time - self._start_time
                estimated = elapsed / percent
                remaining = estimated - elapsed

                self._set_time_label(elapsed, self._elapsed_control)
                self._set_time_label(estimated, self._estimated_control)
                self._set_time_label(remaining, self._remaining_control)

            if value >= self.max or self._user_cancelled:
                self.close()
        else:
            self.progress_bar.setValue(self.progress_bar.value() + value)

            if self._user_cancelled:
                self.close()

        QtGui.QApplication.processEvents()

        return (not self._user_cancelled, False)

    # -------------------------------------------------------------------------
    # Private Interface
    # -------------------------------------------------------------------------

    def reject(self, event):
        self._user_cancelled = True
        self.close()

    def _set_time_label(self, value, control):
        hours = value / 3600
        minutes = (value % 3600) / 60
        seconds = value % 60
        label = "%1u:%02u:%02u" % (hours, minutes, seconds)

        control.setText(label)

    def _create_buttons(self, dialog, layout):
        """ Creates the buttons. """

        if not (self.can_cancel or self.can_ok):
            return

        # Create the button.
        buttons = QtGui.QDialogButtonBox()

        if self.can_cancel:
            buttons.addButton(
                self.cancel_button_label, QtGui.QDialogButtonBox.RejectRole
            )
        if self.can_ok:
            buttons.addButton(QtGui.QDialogButtonBox.Ok)

        # TODO: hookup the buttons to our methods, this may involve subclassing from QDialog

        if self.can_cancel:
            buttons.rejected.connect(dialog.reject)
        if self.can_ok:
            buttons.accepted.connect(dialog.accept)

        layout.addWidget(buttons)

    def _create_label(self, dialog, layout, text):

        dummy = QtGui.QLabel(text, dialog)
        dummy.setAlignment(QtCore.Qt.AlignTop | QtCore.Qt.AlignLeft)

        label = QtGui.QLabel("unknown", dialog)
        label.setAlignment(
            QtCore.Qt.AlignTop | QtCore.Qt.AlignLeft | QtCore.Qt.AlignRight
        )

        sub_layout = QtGui.QHBoxLayout()

        sub_layout.addWidget(dummy)
        sub_layout.addWidget(label)

        layout.addLayout(sub_layout)

        return label

    def _create_gauge(self, dialog, layout):

        self.progress_bar = QtGui.QProgressBar(dialog)
        self.progress_bar.setRange(self.min, self.max)
        layout.addWidget(self.progress_bar)

        if self.show_percent:
            self.progress_bar.setFormat("%p%")
        else:
            self.progress_bar.setFormat("%v")

    def _create_message(self, dialog, layout):
        label = QtGui.QLabel(self.message, dialog)
        label.setAlignment(QtCore.Qt.AlignTop | QtCore.Qt.AlignLeft)
        layout.addWidget(label)
        self._message_control = label

    def _create_percent(self, dialog, parent_sizer):
        if not self.show_percent:
            return

        raise NotImplementedError

    def _create_timer(self, dialog, layout):
        if not self.show_time:
            return

        self._elapsed_control = self._create_label(
            dialog, layout, "Elapsed time : "
        )
        self._estimated_control = self._create_label(
            dialog, layout, "Estimated time : "
        )
        self._remaining_control = self._create_label(
            dialog, layout, "Remaining time : "
        )

    def _create_control(self, parent):
        return QtGui.QDialog(parent)

    def _create(self):
        super(ProgressDialog, self)._create()
        self._create_contents(self.control)

    def _create_contents(self, parent):
        dialog = parent
        layout = QtGui.QVBoxLayout(dialog)
        layout.setContentsMargins(
            self.margin, self.margin, self.margin, self.margin
        )

        # The 'guts' of the dialog.
        self._create_message(dialog, layout)
        self._create_gauge(dialog, layout)
        self._create_timer(dialog, layout)
        self._create_buttons(dialog, layout)

        dialog.setWindowTitle(self.title)

        parent.setLayout(layout)

    # -------------------------------------------------------------------------
    # Trait change handlers
    # -------------------------------------------------------------------------

    def _max_changed(self, new):
        if self.progress_bar is not None:
            self.progress_bar.setMaximum(new)

    def _min_changed(self, new):
        if self.progress_bar is not None:
            self.progress_bar.setMinimum(new)

    def _message_changed(self, new):
        if self._message_control is not None:
            self._message_control.setText(new)



# (C) Copyright 2005-2020 Enthought, Inc., Austin, TX
# All rights reserved.
#
# This software is provided without warranty under the terms of the BSD
# license included in LICENSE.txt and may be redistributed only under
# the conditions described in the aforementioned license. The license
# is also available online at http://www.enthought.com/licenses/BSD.txt
#
# Thanks for using Enthought open source!

"""
Qt-specific tests for the MessageDialog
"""

import contextlib
import unittest

from pyface.api import MessageDialog
from pyface.qt import QtGui
from pyface.ui.qt4.util.gui_test_assistant import GuiTestAssistant


class TestMessageDialog(GuiTestAssistant, unittest.TestCase):
    def test_escape_button_no_details(self):
        dialog = MessageDialog(
            parent=None,
            title=u"Dialog title",
            message=u"Printer on fire",
            informative=u"Your printer is on fire",
            severity=u"error",
            size=(600, 400),
        )

        with self.create_dialog(dialog):
            escape_button = dialog.control.escapeButton()
            ok_button = dialog.control.button(QtGui.QMessageBox.Ok)
            # It's possible for both the above to be None, so double check.
            self.assertIsNotNone(escape_button)
            self.assertIs(escape_button, ok_button)

    def test_escape_button_with_details(self):
        dialog = MessageDialog(
            parent=None,
            title=u"Dialog title",
            message=u"Printer on fire",
            informative=u"Your printer is on fire",
            details=u"Temperature exceeds 1000 degrees",
            severity=u"error",
            size=(600, 400),
        )

        with self.create_dialog(dialog):
            escape_button = dialog.control.escapeButton()
            ok_button = dialog.control.button(QtGui.QMessageBox.Ok)
            # It's possible for both the above to be None, so double check.
            self.assertIsNotNone(escape_button)
            self.assertIs(escape_button, ok_button)

    @contextlib.contextmanager
    def create_dialog(self, dialog):
        """
        Create a dialog, then destroy at the end of a with block.
        """
        with self.event_loop():
            dialog._create()
        try:
            yield
        finally:
            with self.event_loop():
                dialog.destroy()



# (C) Copyright 2005-2020 Enthought, Inc., Austin, TX
# All rights reserved.
#
# This software is provided without warranty under the terms of the BSD
# license included in LICENSE.txt and may be redistributed only under
# the conditions described in the aforementioned license. The license
# is also available online at http://www.enthought.com/licenses/BSD.txt
#
# Thanks for using Enthought open source!

""" Abstract base class for all action manager items. """


from traits.api import Bool, HasTraits, Instance, Str


class ActionManagerItem(HasTraits):
    """ Abstract base class for all action manager items.

    An action manager item represents a contribution to a shared UI resource
    such as a menu bar, menu or tool bar.

    Action manager items know how to add themselves to menu bars, menus and
    tool bars.  In a tool bar a contribution item is represented as a tool or a
    separator.  In a menu bar a contribution item is a menu, and in a menu
    a contribution item is a menu item or separator.

    """

    #: The item's unique identifier ('unique' in this case means unique within
    #: its group)
    id = Str()

    #: The group the item belongs to.
    parent = Instance("pyface.action.api.Group")

    #: Is the item enabled?
    enabled = Bool(True)

    #: Is the item visible?
    visible = Bool(True)

    # ------------------------------------------------------------------------
    # 'ActionManagerItem' interface.
    # ------------------------------------------------------------------------

    def add_to_menu(self, parent, menu, controller):
        """ Adds the item to a menu.

        Parameters
        ----------
        parent : toolkit control
            The parent of the new menu item control.
        menu : toolkit menu
            The menu to add the action item to.
        controller : ActionController instance or None
            The controller to use.
        """
        raise NotImplementedError

    def add_to_toolbar(self, parent, tool_bar, image_cache, controller):
        """ Adds the item to a tool bar.

        Parameters
        ----------
        parent : toolkit control
            The parent of the new menu item control.
        tool_bar : toolkit toolbar
            The toolbar to add the action item to.
        image_cache : ImageCache instance
            The image cache for resized images.
        controller : ActionController instance or None
            The controller to use.
        show_labels : bool
            Should the toolbar item show a label.
        """
        raise NotImplementedError



# (C) Copyright 2005-2020 Enthought, Inc., Austin, TX
# All rights reserved.
#
# This software is provided without warranty under the terms of the BSD
# license included in LICENSE.txt and may be redistributed only under
# the conditions described in the aforementioned license. The license
# is also available online at http://www.enthought.com/licenses/BSD.txt
#
# Thanks for using Enthought open source!
"""
Event-loop based timers that perform actions periodically.

Note that if a timer goes out of scope without a reference to being saved,
there is nothing keeping the underlying toolkit timer alive and it will be
garbage collected, meaning that the timer will stop firing (or indeed, may
never fire).
"""

from pyface.toolkit import toolkit_object
from pyface.timer.i_timer import MCallbackTimer, MEventTimer

PyfaceTimer = toolkit_object("timer.timer:PyfaceTimer")


class EventTimer(MEventTimer, PyfaceTimer):
    pass


class CallbackTimer(MCallbackTimer, PyfaceTimer):
    pass


class Timer(CallbackTimer):
    """ Subclass of CallbackTimer that matches the old API """

    def __init__(self, millisecs, callable, *args, **kwargs):
        """ Initialize and start the timer.

        Initialize instance to invoke the given `callable` with given
        arguments and keyword args after every `millisecs` (milliseconds).
        """
        interval = millisecs / 1000.0
        super(Timer, self).__init__(
            interval=interval, callback=callable, args=args, kwargs=kwargs
        )
        self.start()

    def Notify(self):
        """ Alias for `perform` to match old API.
        """
        self.perform()

    def Start(self, millisecs=None):
        """ Alias for `start` to match old API.
        """
        if millisecs is not None:
            self.interval = millisecs / 1000.0

        self.start()

    def Stop(self):
        """ Alias for `stop` to match old API.
        """
        self.stop()

    def IsRunning(self):
        """ Alias for is_running property to match old API.
        """
        return self._active



# (C) Copyright 2005-2020 Enthought, Inc., Austin, TX
# All rights reserved.
#
# This software is provided without warranty under the terms of the BSD
# license included in LICENSE.txt and may be redistributed only under
# the conditions described in the aforementioned license. The license
# is also available online at http://www.enthought.com/licenses/BSD.txt
#
# Thanks for using Enthought open source!
""" Object with views for naming or renaming a user perspective. """


from traits.api import Bool, HasTraits, Constant, String
from traitsui.api import View, Item, VGroup


# Trait definitions --------------------------------------------------------

# Define a trait which can not be the empty string:
NotEmptyString = String(minlen=1)


class UserPerspectiveName(HasTraits):
    """ Object with views for naming or renaming a user perspective. """

    # ------------------------------------------------------------------------
    # 'UserPerspectiveName' interface.
    # ------------------------------------------------------------------------

    # The name of the new user perspective.
    name = NotEmptyString

    # Should the editor area be shown in this perpsective?
    show_editor_area = Bool(True)

    # Help notes when creating a new view.
    new_help = Constant(
        """Note:
 - The new perspective will initially be empty.
 - Add new views to the perspective by selecting
   them from the 'View' menu.
 - Drag the notebook tabs and splitter bars to
   arrange the views within the perspective."""
    )

    # Traits views ---------------------------------------------------------

    new_view = View(
        VGroup(
            VGroup("name", "show_editor_area"),
            VGroup("_", Item("new_help", style="readonly"), show_labels=False),
        ),
        title="New User Perspective",
        id="envisage.workbench.action."
        "new_user_perspective_action.UserPerspectiveName",
        buttons=["OK", "Cancel"],
        kind="livemodal",
        width=300,
    )

    save_as_view = View(
        "name",
        title="Save User Perspective As",
        id="envisage.workbench.action."
        "save_as_user_perspective_action.UserPerspectiveName",
        buttons=["OK", "Cancel"],
        kind="livemodal",
        width=300,
    )

    rename_view = View(
        "name",
        title="Rename User Perspective",
        id="envisage.workbench.action."
        "rename_user_perspective_action.UserPerspectiveName",
        buttons=["OK", "Cancel"],
        kind="livemodal",
        width=300,
    )



# (C) Copyright 2005-2020 Enthought, Inc., Austin, TX
# All rights reserved.
#
# This software is provided without warranty under the terms of the BSD
# license included in LICENSE.txt and may be redistributed only under
# the conditions described in the aforementioned license. The license
# is also available online at http://www.enthought.com/licenses/BSD.txt
#
# Thanks for using Enthought open source!

""" The base class for all actions. """

from functools import partial


from traits.api import Bool, Callable, Enum, HasTraits, Str
from traits.api import Str

from pyface.ui_traits import Image


class Action(HasTraits):
    """ The base class for all actions.

    An action is the non-UI side of a command which can be triggered by the end
    user.  Actions are typically associated with buttons, menu items and tool
    bar tools.

    When the user triggers the command via the UI, the action's 'perform'
    method is invoked to do the actual work.

    """

    # 'Action' interface ---------------------------------------------------

    #: Keyboard accelerator (by default the action has NO accelerator).
    accelerator = Str()

    #: Is the action checked?  This is only relevant if the action style is
    #: 'radio' or 'toggle'.
    checked = Bool(False)

    #: A longer description of the action (used for context sensitive help etc).
    #: If no description is specified, the tooltip is used instead (and if there
    #: is no tooltip, then well, maybe you just hate your users ;^).
    description = Str()

    #: Is the action enabled?
    enabled = Bool(True)

    #: Is the action visible?
    visible = Bool(True)

    #: The action's unique identifier (may be None).
    id = Str()

    #: The action's image (displayed on tool bar tools etc).
    image = Image

    #: The action's name (displayed on menus/tool bar tools etc).
    name = Str()

    #: An (optional) callable that will be invoked when the action is performed.
    on_perform = Callable

    #: The action's style.
    style = Enum("push", "radio", "toggle", "widget")

    #: A short description of the action used for tooltip text etc.
    tooltip = Str()

    #: An (optional) callable to create the toolkit control for widget style.
    control_factory = Callable

    # ------------------------------------------------------------------------
    # 'Action' interface.
    # ------------------------------------------------------------------------

    # Initializers ---------------------------------------------------------

    def _id_default(self):
        """ Initializes the 'id' trait.

        The default is the ``name`` trait.
        """
        return self.name

    # Methods -------------------------------------------------------------#

    def create_control(self, parent):
        """ Called when creating a "widget" style action.

        By default this will call whatever callable is supplied via the
        'control_factory' trait which is a callable that should take the parent
        control and the action as arguments and return an appropriate toolkit
        control.  Some operating systems (Mac OS in particular) may limit what
        widgets can be displayed in menus.

        This method is only used when the 'style' is "widget" and is ignored by
        other styles.

        Parameters
        ----------
        parent : toolkit control
            The toolkit control, usually a toolbar.

        Returns
        -------
        control : toolkit control
            A toolkit control or None.
        """
        if self.style == "widget" and self.control_factory is not None:
            return self.control_factory(parent, self)
        return None

    def destroy(self):
        """ Called when the action is no longer required.

        By default this method does nothing, but this would be a great place to
        unhook trait listeners etc.
        """

    def perform(self, event):
        """ Performs the action.

        Parameters
        ----------
        event : ActionEvent instance
            The event which triggered the action.
        """
        if self.on_perform is not None:
            self.on_perform()

    @classmethod
    def factory(cls, *args, **kwargs):
        """ Create a factory for an action with the given arguments.

        This is particularly useful for passing context to Tasks schema
        additions.
        """
        return partial(cls, *args, **kwargs)



# (C) Copyright 2005-2020 Enthought, Inc., Austin, TX
# All rights reserved.
#
# This software is provided without warranty under the terms of the BSD
# license included in LICENSE.txt and may be redistributed only under
# the conditions described in the aforementioned license. The license
# is also available online at http://www.enthought.com/licenses/BSD.txt
#
# Thanks for using Enthought open source!

""" A description of a row in a grid. """


from traits.api import HasTraits


class GridRow(HasTraits):
    """ A description of a row in a grid. """

    def __init__(self, row_data):
        """ Create a new row. """

        self.__dict__.update(row_data)

        return



# (C) Copyright 2005-2020 Enthought, Inc., Austin, TX
# All rights reserved.
#
# This software is provided without warranty under the terms of the BSD
# license included in LICENSE.txt and may be redistributed only under
# the conditions described in the aforementioned license. The license
# is also available online at http://www.enthought.com/licenses/BSD.txt
#
# Thanks for using Enthought open source!
""" The menu bar manager for Envisage workbench windows. """


from pyface.action.api import MenuBarManager as BaseMenuBarManager
from traits.api import Instance


from .action_controller import ActionController


class MenuBarManager(BaseMenuBarManager):
    """ The menu bar manager for Envisage workbench windows. """

    # 'MenuBarManager' interface -------------------------------------------

    # The workbench window that we are the menu bar manager for.
    window = Instance("pyface.workbench.api.WorkbenchWindow")

    # ------------------------------------------------------------------------
    # 'MenuBarManager' interface.
    # ------------------------------------------------------------------------

    def create_menu_bar(self, parent):
        """ Creates a menu bar representation of the manager. """

        # The controller handles the invocation of every action.
        controller = ActionController(window=self.window)

        menu_bar = super(MenuBarManager, self).create_menu_bar(
            parent, controller=controller
        )

        return menu_bar



# (C) Copyright 2005-2020 Enthought, Inc., Austin, TX
# All rights reserved.
#
# This software is provided without warranty under the terms of the BSD
# license included in LICENSE.txt and may be redistributed only under
# the conditions described in the aforementioned license. The license
# is also available online at http://www.enthought.com/licenses/BSD.txt
#
# Thanks for using Enthought open source!

""" The Qt-specific implementation of the text field class """


from traits.api import Trait, provides

from pyface.fields.i_text_field import ITextField, MTextField
from pyface.qt.QtGui import QLineEdit
from .field import Field


ECHO_TO_QT_ECHO_MODE = {
    "normal": QLineEdit.Normal,
    "password": QLineEdit.Password,
    "none": QLineEdit.NoEcho,
    "when_editing": QLineEdit.PasswordEchoOnEdit,
}
QT_ECHO_MODE_TO_ECHO = {
    value: key for key, value in ECHO_TO_QT_ECHO_MODE.items()
}

# mapped trait for Qt line edit echo modes
Echo = Trait("normal", ECHO_TO_QT_ECHO_MODE)


@provides(ITextField)
class TextField(MTextField, Field):
    """ The Qt-specific implementation of the text field class """

    #: Display typed text, or one of several hidden "password" modes.
    echo = Echo

    # ------------------------------------------------------------------------
    # IWidget interface
    # ------------------------------------------------------------------------

    def _create_control(self, parent):
        """ Create the toolkit-specific control that represents the widget. """
        control = QLineEdit(parent)
        return control

    # ------------------------------------------------------------------------
    # Private interface
    # ------------------------------------------------------------------------

    def _get_control_value(self):
        """ Toolkit specific method to get the control's value. """
        return self.control.text()

    def _set_control_value(self, value):
        """ Toolkit specific method to set the control's value. """
        self.control.setText(value)
        # fire update
        if self.update_text == "editing_finished":
            self.control.editingFinished.emit()
        else:
            self.control.textEdited.emit(value)

    def _observe_control_value(self, remove=False):
        """ Toolkit specific method to change the control value observer. """
        if remove:
            self.control.textEdited.disconnect(self._update_value)
        else:
            self.control.textEdited.connect(self._update_value)

    def _get_control_placeholder(self):
        """ Toolkit specific method to set the control's placeholder. """
        return self.control.placeholderText()

    def _set_control_placeholder(self, placeholder):
        """ Toolkit specific method to set the control's placeholder. """
        self.control.setPlaceholderText(placeholder)

    def _get_control_echo(self):
        """ Toolkit specific method to get the control's echo. """
        return QT_ECHO_MODE_TO_ECHO[self.control.echoMode()]

    def _set_control_echo(self, echo):
        """ Toolkit specific method to set the control's echo. """
        self.control.setEchoMode(ECHO_TO_QT_ECHO_MODE[echo])

    def _get_control_read_only(self):
        """ Toolkit specific method to get the control's read_only state. """
        return self.control.isReadOnly()

    def _set_control_read_only(self, read_only):
        """ Toolkit specific method to set the control's read_only state. """
        self.control.setReadOnly(read_only)

    def _observe_control_editing_finished(self, remove=False):
        """ Change observation of whether editing is finished. """
        if remove:
            self.control.editingFinished.disconnect(self._editing_finished)
        else:
            self.control.editingFinished.connect(self._editing_finished)



# (C) Copyright 2005-2020 Enthought, Inc., Austin, TX
# All rights reserved.
#
# This software is provided without warranty under the terms of the BSD
# license included in LICENSE.txt and may be redistributed only under
# the conditions described in the aforementioned license. The license
# is also available online at http://www.enthought.com/licenses/BSD.txt
#
# Thanks for using Enthought open source!


import os
from shutil import rmtree
from tempfile import mkdtemp
import unittest

from traits.api import Bool, on_trait_change

from ..application_window import ApplicationWindow
from ..gui_application import GUIApplication
from ..toolkit import toolkit_object

GuiTestAssistant = toolkit_object("util.gui_test_assistant:GuiTestAssistant")
no_gui_test_assistant = GuiTestAssistant.__name__ == "Unimplemented"

EVENTS = [
    "starting",
    "started",
    "application_initialized",
    "stopping",
    "stopped",
]


class TestingApp(GUIApplication):

    #: Whether the app should start cleanly.
    start_cleanly = Bool(True)

    #: Whether the app should stop cleanly.
    stop_cleanly = Bool(True)

    #: Whether to try invoking exit method.
    do_exit = Bool(False)

    #: Whether the exit should be invoked as an error exit.
    error_exit = Bool(False)

    #: Whether to try force the exit (ie. ignore vetoes).
    force_exit = Bool(False)

    #: Whether to veto a call to the exit method.
    veto_exit = Bool(False)

    #: Whether to veto a opening a window.
    veto_open_window = Bool(False)

    #: Whether to veto a closing a window.
    veto_close_window = Bool(False)

    #: Whether or not a call to the open a window was vetoed.
    window_open_vetoed = Bool(False)

    #: Whether or not a call to the exit method was vetoed.
    exit_vetoed = Bool(False)

    #: Whether exit preparation happened.
    exit_prepared = Bool(False)

    #: Whether exit preparation raises an error.
    exit_prepared_error = Bool(False)

    def start(self):
        if not self.start_cleanly:
            return False
        super(TestingApp, self).start()

        window = self.windows[0]
        window.on_trait_change(self._on_window_closing, "closing")
        return True

    def stop(self):
        super(TestingApp, self).stop()
        return self.stop_cleanly

    def _on_window_closing(self, window, trait, old, new):
        if self.veto_close_window and not self.exit_vetoed:
            new.veto = True
            self.exit_vetoed = True

    def _application_initialized_fired(self):
        self.window_open_vetoed = (
            len(self.windows) > 0 and self.windows[0].control is None
        )

    def _exiting_fired(self, event):
        event.veto = self.veto_exit
        self.exit_vetoed = self.veto_exit

    def _prepare_exit(self):
        super(TestingApp, self)._prepare_exit()
        if not self.exit_vetoed:
            self.exit_prepared = True
        if self.exit_prepared_error:
            raise Exception("Exit preparation failed")

    @on_trait_change("windows:opening")
    def _on_activate_window(self, event):
        if self.veto_open_window:
            event.veto = self.veto_open_window


@unittest.skipIf(no_gui_test_assistant, "No GuiTestAssistant")
class TestGUIApplication(unittest.TestCase, GuiTestAssistant):
    def setUp(self):
        GuiTestAssistant.setUp(self)
        self.application_events = []

        if toolkit_object.toolkit == "wx":
            import wx

            self.event_loop()
            wx.GetApp().DeletePendingEvents()
        else:
            self.event_loop()

    def tearDown(self):
        GuiTestAssistant.tearDown(self)

    def event_listener(self, event):
        self.application_events.append(event)

    def connect_listeners(self, app):
        for event in EVENTS:
            app.on_trait_change(self.event_listener, event)

    def test_defaults(self):
        from traits.etsconfig.api import ETSConfig

        app = GUIApplication()

        self.assertEqual(app.home, ETSConfig.application_home)
        self.assertEqual(app.user_data, ETSConfig.user_data)
        self.assertEqual(app.company, ETSConfig.company)

    def test_initialize_application_home(self):
        dirpath = mkdtemp()
        home = os.path.join(dirpath, "test")
        app = GUIApplication(home=home)

        app.initialize_application_home()
        try:
            self.assertTrue(os.path.exists(home))
        finally:
            rmtree(dirpath)

    def test_lifecycle(self):

        app = GUIApplication()
        self.connect_listeners(app)
        window = ApplicationWindow()
        app.on_trait_change(lambda: app.add_window(window), "started")

        with self.assertMultiTraitChanges([app], EVENTS, []):
            self.gui.invoke_after(1000, app.exit)
            result = app.run()

        self.assertTrue(result)
        event_order = [event.event_type for event in self.application_events]
        self.assertEqual(event_order, EVENTS)
        self.assertEqual(app.windows, [])

    def test_exit_prepare_error(self):
        app = TestingApp(exit_prepared_error=True)
        self.connect_listeners(app)

        with self.assertMultiTraitChanges([app], EVENTS, []):
            self.gui.invoke_after(1000, app.exit)
            result = app.run()

        self.assertTrue(result)
        self.assertFalse(app.exit_vetoed)
        self.assertTrue(app.exit_prepared)
        event_order = [event.event_type for event in self.application_events]
        self.assertEqual(event_order, EVENTS)
        self.assertEqual(app.windows, [])

    def test_veto_exit(self):
        app = TestingApp(veto_exit=True)
        self.connect_listeners(app)

        with self.assertMultiTraitChanges([app], EVENTS, []):
            self.gui.invoke_after(1000, app.exit)
            self.gui.invoke_after(2000, app.exit, force=True)
            result = app.run()

        self.assertTrue(result)
        self.assertTrue(app.exit_vetoed)
        self.assertFalse(app.exit_prepared)
        event_order = [event.event_type for event in self.application_events]
        self.assertEqual(event_order, EVENTS)
        self.assertEqual(app.windows, [])

    def test_veto_open_window(self):
        app = TestingApp(veto_open_window=True)
        self.connect_listeners(app)

        with self.assertMultiTraitChanges([app], EVENTS, []):
            self.gui.invoke_after(1000, app.exit)
            result = app.run()

        self.assertTrue(result)
        self.assertTrue(app.window_open_vetoed)
        self.assertFalse(app.exit_vetoed)
        self.assertTrue(app.exit_prepared)
        event_order = [event.event_type for event in self.application_events]
        self.assertEqual(event_order, EVENTS)
        self.assertEqual(app.windows, [])

    def test_veto_close_window(self):
        app = TestingApp(veto_close_window=True)
        self.connect_listeners(app)

        with self.assertMultiTraitChanges([app], EVENTS, []):
            self.gui.invoke_after(1000, app.exit)
            self.gui.invoke_after(2000, app.exit, force=True)
            result = app.run()

        self.assertTrue(result)
        self.assertTrue(app.exit_vetoed)
        self.assertFalse(app.exit_prepared)
        event_order = [event.event_type for event in self.application_events]
        self.assertEqual(event_order, EVENTS)
        self.assertEqual(app.windows, [])

    def test_force_exit(self):
        app = TestingApp(do_exit=True, force_exit=True, veto_exit=True)
        self.connect_listeners(app)

        with self.assertMultiTraitChanges([app], EVENTS, []):
            self.gui.invoke_after(100, app.exit, True)
            result = app.run()

        self.assertTrue(result)
        self.assertFalse(app.exit_vetoed)
        self.assertTrue(app.exit_prepared)
        event_order = [event.event_type for event in self.application_events]
        self.assertEqual(event_order, EVENTS)
        self.assertEqual(app.windows, [])

    def test_force_exit_close_veto(self):
        app = TestingApp(do_exit=True, force_exit=True, veto_close_window=True)
        self.connect_listeners(app)

        with self.assertMultiTraitChanges([app], EVENTS, []):
            self.gui.invoke_after(1000, app.exit, True)
            result = app.run()

        self.assertTrue(result)
        self.assertFalse(app.exit_vetoed)
        self.assertTrue(app.exit_prepared)
        event_order = [event.event_type for event in self.application_events]
        self.assertEqual(event_order, EVENTS)
        self.assertEqual(app.windows, [])

    def test_bad_start(self):
        app = TestingApp(start_cleanly=False)
        self.connect_listeners(app)

        with self.assertMultiTraitChanges([app], EVENTS[:1], EVENTS[1:]):
            result = app.run()

        self.assertFalse(result)
        event_order = [event.event_type for event in self.application_events]
        self.assertEqual(event_order, EVENTS[:1])
        self.assertEqual(app.windows, [])

    def test_bad_stop(self):
        app = TestingApp(stop_cleanly=False)
        self.connect_listeners(app)

        with self.assertMultiTraitChanges([app], EVENTS[:-1], EVENTS[-1:]):
            self.gui.invoke_after(1000, app.exit, True)
            result = app.run()

        self.assertFalse(result)
        event_order = [event.event_type for event in self.application_events]
        self.assertEqual(event_order, EVENTS[:-1])
        self.assertEqual(app.windows, [])



# (C) Copyright 2005-2020 Enthought, Inc., Austin, TX
# All rights reserved.
#
# This software is provided without warranty under the terms of the BSD
# license included in LICENSE.txt and may be redistributed only under
# the conditions described in the aforementioned license. The license
# is also available online at http://www.enthought.com/licenses/BSD.txt
#
# Thanks for using Enthought open source!


from traits.testing.unittest_tools import UnittestTools

from pyface.action.api import Action, MenuManager
from pyface.gui import GUI
from pyface.window import Window


class FieldMixin(UnittestTools):
    """ Mixin which provides standard methods for all fields. """

    def setUp(self):
        self.gui = GUI()

        self.parent = Window()
        self.parent._create()
        self.addCleanup(self._destroy_parent)
        self.gui.process_events()

        self.widget = self._create_widget()

        self.parent.open()
        self.gui.process_events()

    def _create_widget(self):
        raise NotImplementedError()

    def _create_widget_control(self):
        self.widget._create()
        self.addCleanup(self._destroy_widget)
        self.widget.show(True)
        self.gui.process_events()

    def _destroy_parent(self):
        self.parent.destroy()
        self.gui.process_events()
        self.parent = None

    def _destroy_widget(self):
        self.widget.destroy()
        self.gui.process_events()
        self.widget = None

    # Tests ------------------------------------------------------------------

    def test_field_tooltip(self):
        self._create_widget_control()
        self.widget.tooltip = "New tooltip."
        self.gui.process_events()

        self.assertEqual(self.widget._get_control_tooltip(), "New tooltip.")

    def test_field_menu(self):
        self._create_widget_control()
        self.widget.menu = MenuManager(Action(name="Test"), name="Test")
        self.gui.process_events()



# (C) Copyright 2005-2020 Enthought, Inc., Austin, TX
# All rights reserved.
#
# This software is provided without warranty under the terms of the BSD
# license included in LICENSE.txt and may be redistributed only under
# the conditions described in the aforementioned license. The license
# is also available online at http://www.enthought.com/licenses/BSD.txt
#
# Thanks for using Enthought open source!
# (C) Copyright 2007 Riverbank Computing Limited
# This software is provided without warranty under the terms of the BSD license.
# However, when used with the GPL version of PyQt the additional terms described in the PyQt GPL exception also apply


from pyface.qt import QtGui


from traits.api import HasTraits, provides


from pyface.i_image_cache import IImageCache, MImageCache


@provides(IImageCache)
class ImageCache(MImageCache, HasTraits):
    """ The toolkit specific implementation of an ImageCache.  See the
    IImageCache interface for the API documentation.
    """

    # ------------------------------------------------------------------------
    # 'object' interface.
    # ------------------------------------------------------------------------

    def __init__(self, width, height):
        self._width = width
        self._height = height

    # ------------------------------------------------------------------------
    # 'ImageCache' interface.
    # ------------------------------------------------------------------------

    def get_image(self, filename):
        image = QtGui.QPixmapCache.find(filename)

        if image is not None:
            scaled = self._qt4_scale(image)

            if scaled is not image:
                # The Qt cache is application wide so we only keep the last
                # size asked for.
                QtGui.QPixmapCache.remove(filename)
                QtGui.QPixmapCache.insert(filename, scaled)
        else:
            # Load the image from the file and add it to the cache.
            image = QtGui.QPixmap(filename)
            scaled = self._qt4_scale(image)
            QtGui.QPixmapCache.insert(filename, scaled)

        return scaled

    # Qt doesn't distinguish between bitmaps and images.
    get_bitmap = get_image

    # ------------------------------------------------------------------------
    # Private 'ImageCache' interface.
    # ------------------------------------------------------------------------

    def _qt4_scale(self, image):
        """ Scales the given image if necessary. """

        # Although Qt won't scale the image if it doesn't need to, it will make
        # a deep copy which we don't need.
        if image.width() != self._width or image.height() != self._height:
            image = image.scaled(self._width, self._height)

        return image



# (C) Copyright 2005-2020 Enthought, Inc., Austin, TX
# All rights reserved.
#
# This software is provided without warranty under the terms of the BSD
# license included in LICENSE.txt and may be redistributed only under
# the conditions described in the aforementioned license. The license
# is also available online at http://www.enthought.com/licenses/BSD.txt
#
# Thanks for using Enthought open source!

""" A page in a wizard. """


import wx


from traits.api import Bool, HasTraits, provides, Str, Tuple, Str
from pyface.api import HeadingText
from pyface.wizard.i_wizard_page import IWizardPage, MWizardPage


@provides(IWizardPage)
class WizardPage(MWizardPage, HasTraits):
    """ The toolkit specific implementation of a WizardPage.

    See the IWizardPage interface for the API documentation.

    """

    # 'IWizardPage' interface ---------------------------------------------#

    id = Str()

    next_id = Str()

    last_page = Bool(False)

    complete = Bool(False)

    heading = Str()

    subheading = Str()

    size = Tuple()

    # ------------------------------------------------------------------------
    # 'IWizardPage' interface.
    # ------------------------------------------------------------------------

    def create_page(self, parent):
        """ Creates the wizard page. """

        # FIXME: implement support for the size trait.

        panel = wx.Panel(parent, -1, style=wx.CLIP_CHILDREN)
        sizer = wx.BoxSizer(wx.VERTICAL)
        panel.SetSizer(sizer)
        panel.SetAutoLayout(True)

        # The 'pretty' heading ;^)
        if len(self.heading) > 0:
            title = HeadingText(panel, text=self.heading)
            sizer.Add(title.control, 0, wx.EXPAND | wx.BOTTOM, 5)

        if len(self.subheading) > 0:
            subtitle = wx.StaticText(panel, -1, self.subheading)
            sizer.Add(subtitle, 0, wx.EXPAND | wx.BOTTOM, 5)

        # The page content.
        content = self._create_page_content(panel)
        sizer.Add(content, 1, wx.EXPAND)

        return panel

    # ------------------------------------------------------------------------
    # Protected 'IWizardPage' interface.
    # ------------------------------------------------------------------------

    def _create_page_content(self, parent):
        """ Creates the actual page content. """

        # Dummy implementation - override!
        panel = wx.Panel(parent, -1, style=wx.CLIP_CHILDREN)
        panel.SetBackgroundColour("yellow")

        return panel



# (C) Copyright 2005-2020 Enthought, Inc., Austin, TX
# All rights reserved.
#
# This software is provided without warranty under the terms of the BSD
# license included in LICENSE.txt and may be redistributed only under
# the conditions described in the aforementioned license. The license
# is also available online at http://www.enthought.com/licenses/BSD.txt
#
# Thanks for using Enthought open source!
# (C) Copyright 2007 Riverbank Computing Limited
# This software is provided without warranty under the terms of the BSD license.
# However, when used with the GPL version of PyQt the additional terms described in the PyQt GPL exception also apply


import os


from pyface.qt import QtCore, QtGui


from traits.api import Enum, Int, List, provides, Str


from pyface.i_file_dialog import IFileDialog, MFileDialog
from .dialog import Dialog


@provides(IFileDialog)
class FileDialog(MFileDialog, Dialog):
    """ The toolkit specific implementation of a FileDialog.  See the
    IFileDialog interface for the API documentation.
    """

    # 'IFileDialog' interface ---------------------------------------------#

    action = Enum("open", "open files", "save as")

    default_directory = Str()

    default_filename = Str()

    default_path = Str()

    directory = Str()

    filename = Str()

    path = Str()

    paths = List(Str)

    wildcard = Str()

    wildcard_index = Int(0)

    # ------------------------------------------------------------------------
    # 'MFileDialog' *CLASS* interface.
    # ------------------------------------------------------------------------

    # In Windows, Qt needs only a * while wx needs a *.*
    WILDCARD_ALL = "All files (*)|*"

    @classmethod
    def create_wildcard(cls, description, extension):
        """ Creates a wildcard for a given extension. """

        if isinstance(extension, str):
            pattern = extension

        else:
            pattern = " ".join(extension)

        return "%s (%s)|%s|" % (description, pattern, pattern)

    # ------------------------------------------------------------------------
    # Protected 'IDialog' interface.
    # ------------------------------------------------------------------------

    def _create_contents(self, parent):
        # In PyQt this is a canned dialog.
        pass

    # ------------------------------------------------------------------------
    # 'IWindow' interface.
    # ------------------------------------------------------------------------

    def close(self):
        # Get the path of the chosen directory.
        files = self.control.selectedFiles()

        if files:
            self.path = str(files[0])
            self.paths = [str(file) for file in files]
        else:
            self.path = ""
            self.paths = [""]

        # Extract the directory and filename.
        self.directory, self.filename = os.path.split(self.path)

        # Get the index of the selected filter.
        self.wildcard_index = self.control.nameFilters().index(
            self.control.selectedNameFilter()
        )

        # Let the window close as normal.
        super(FileDialog, self).close()

    # ------------------------------------------------------------------------
    # Protected 'IWidget' interface.
    # ------------------------------------------------------------------------

    def _create_control(self, parent):
        # If the caller provided a default path instead of a default directory
        # and filename, split the path into it directory and filename
        # components.
        if (
            len(self.default_path) != 0
            and len(self.default_directory) == 0
            and len(self.default_filename) == 0
        ):
            default_directory, default_filename = os.path.split(
                self.default_path
            )
        else:
            default_directory = self.default_directory
            default_filename = self.default_filename

        # Convert the filter.
        filters = []
        for filter_list in self.wildcard.split("|")[::2]:
            # Qt uses spaces instead of semicolons for extension separators
            filter_list = filter_list.replace(";", " ")
            filters.append(filter_list)

        # Set the default directory.
        if not default_directory:
            default_directory = QtCore.QDir.currentPath()

        dlg = QtGui.QFileDialog(parent, self.title, default_directory)
        dlg.setViewMode(QtGui.QFileDialog.Detail)
        dlg.selectFile(default_filename)
        dlg.setNameFilters(filters)

        if self.wildcard_index < len(filters):
            dlg.selectNameFilter(filters[self.wildcard_index])

        if self.action == "open":
            dlg.setAcceptMode(QtGui.QFileDialog.AcceptOpen)
            dlg.setFileMode(QtGui.QFileDialog.ExistingFile)
        elif self.action == "open files":
            dlg.setAcceptMode(QtGui.QFileDialog.AcceptOpen)
            dlg.setFileMode(QtGui.QFileDialog.ExistingFiles)
        else:
            dlg.setAcceptMode(QtGui.QFileDialog.AcceptSave)
            dlg.setFileMode(QtGui.QFileDialog.AnyFile)

        return dlg

    # ------------------------------------------------------------------------
    # Trait handlers.
    # ------------------------------------------------------------------------

    def _wildcard_default(self):
        """ Return the default wildcard. """

        return self.WILDCARD_ALL



# (C) Copyright 2005-2020 Enthought, Inc., Austin, TX
# All rights reserved.
#
# This software is provided without warranty under the terms of the BSD
# license included in LICENSE.txt and may be redistributed only under
# the conditions described in the aforementioned license. The license
# is also available online at http://www.enthought.com/licenses/BSD.txt
#
# Thanks for using Enthought open source!


import os
import unittest

from traits.api import DefaultValue, HasTraits, TraitError
from traits.testing.unittest_tools import UnittestTools

from ..i_image_resource import IImageResource
from ..image_resource import ImageResource
from ..ui_traits import (
    Border,
    HasBorder,
    HasMargin,
    Image,
    Margin,
    image_resource_cache,
    image_bitmap_cache,
)


IMAGE_PATH = os.path.join(os.path.dirname(__file__), "images", "core.png")


class ImageClass(HasTraits):

    image = Image


class HasMarginClass(HasTraits):

    margin = HasMargin


class HasBorderClass(HasTraits):

    border = HasBorder


class TestImageTrait(unittest.TestCase, UnittestTools):
    def setUp(self):
        # clear all cached images
        image_resource_cache.clear()
        image_bitmap_cache.clear()
        # clear cached "not found" image
        ImageResource._image_not_found = None

    def test_defaults(self):
        image_class = ImageClass()

        self.assertIsNone(image_class.image)

    def test_init_local_image(self):
        from pyface.image_resource import ImageResource

        image_class = ImageClass(image=ImageResource("core.png"))

        self.assertIsInstance(image_class.image, ImageResource)
        self.assertEqual(image_class.image.name, "core.png")
        self.assertEqual(
            image_class.image.absolute_path, os.path.abspath(IMAGE_PATH)
        )

    def test_init_pyface_image(self):
        from pyface.image_resource import ImageResource

        image_class = ImageClass(image="about")
        im = image_class.image.create_image()

        self.assertIsInstance(image_class.image, ImageResource)
        self.assertEqual(image_class.image.name, "about")
        self.assertIsNone(image_class.image._image_not_found)
        self.assertIsNotNone(image_class.image._ref.data)

    def test_init_pyface_image_library(self):
        from pyface.image_resource import ImageResource

        image_class = ImageClass(image="@icons:dialog-warning")

        self.assertIsInstance(image_class.image, ImageResource)
        self.assertEqual(image_class.image.name, "dialog-warning.png")
        self.assertIsNone(image_class.image._image_not_found)
        self.assertEqual(
            image_class.image._ref.file_name, "dialog-warning.png"
        )
        self.assertEqual(image_class.image._ref.volume_name, "icons")


class TestMargin(unittest.TestCase):
    def test_defaults(self):
        margin = Margin()
        self.assertEqual(margin.top, 0)
        self.assertEqual(margin.bottom, 0)
        self.assertEqual(margin.left, 0)
        self.assertEqual(margin.right, 0)

    def test_init_one_arg(self):
        margin = Margin(4)
        self.assertEqual(margin.top, 4)
        self.assertEqual(margin.bottom, 4)
        self.assertEqual(margin.left, 4)
        self.assertEqual(margin.right, 4)

    def test_init_two_args(self):
        margin = Margin(4, 2)
        self.assertEqual(margin.top, 2)
        self.assertEqual(margin.bottom, 2)
        self.assertEqual(margin.left, 4)
        self.assertEqual(margin.right, 4)

    def test_init_four_args(self):
        margin = Margin(4, 2, 3, 1)
        self.assertEqual(margin.top, 3)
        self.assertEqual(margin.bottom, 1)
        self.assertEqual(margin.left, 4)
        self.assertEqual(margin.right, 2)


class TestHasMargin(unittest.TestCase, UnittestTools):
    def test_defaults(self):
        has_margin = HasMarginClass()
        margin = has_margin.margin

        self.assertEqual(margin.top, 0)
        self.assertEqual(margin.bottom, 0)
        self.assertEqual(margin.left, 0)
        self.assertEqual(margin.right, 0)

    def test_unspecified_default(self):
        trait = HasMargin()
        trait.default_value_type = DefaultValue.unspecified

        (dvt, dv) = trait.get_default_value()

        self.assertEqual(dvt, DefaultValue.callable_and_args)
        self.assertEqual(
            dv, (Margin, (), {"top": 0, "bottom": 0, "left": 0, "right": 0})
        )

    def test_default_int(self):
        class HasMarginClass(HasTraits):

            margin = HasMargin(4)

        has_margin = HasMarginClass()
        margin = has_margin.margin

        self.assertEqual(margin.top, 4)
        self.assertEqual(margin.bottom, 4)
        self.assertEqual(margin.left, 4)
        self.assertEqual(margin.right, 4)

    def test_default_one_tuple(self):
        class HasMarginClass(HasTraits):

            margin = HasMargin((4,))

        has_margin = HasMarginClass()
        margin = has_margin.margin

        self.assertEqual(margin.top, 4)
        self.assertEqual(margin.bottom, 4)
        self.assertEqual(margin.left, 4)
        self.assertEqual(margin.right, 4)

    def test_default_two_tuple(self):
        class HasMarginClass(HasTraits):

            margin = HasMargin((4, 2))

        has_margin = HasMarginClass()
        margin = has_margin.margin

        self.assertEqual(margin.top, 2)
        self.assertEqual(margin.bottom, 2)
        self.assertEqual(margin.left, 4)
        self.assertEqual(margin.right, 4)

    def test_default_four_tuple(self):
        class HasMarginClass(HasTraits):

            margin = HasMargin((4, 2, 3, 1))

        has_margin = HasMarginClass()
        margin = has_margin.margin

        self.assertEqual(margin.top, 3)
        self.assertEqual(margin.bottom, 1)
        self.assertEqual(margin.left, 4)
        self.assertEqual(margin.right, 2)

    def test_default_margin(self):
        m = Margin(left=4, right=2, top=3, bottom=1)

        class HasMarginClass(HasTraits):

            margin = HasMargin(m)

        has_margin = HasMarginClass()
        margin = has_margin.margin

        self.assertEqual(margin.top, 3)
        self.assertEqual(margin.bottom, 1)
        self.assertEqual(margin.left, 4)
        self.assertEqual(margin.right, 2)

    def test_init_int(self):
        has_margin = HasMarginClass(margin=4)
        margin = has_margin.margin

        self.assertEqual(margin.top, 4)
        self.assertEqual(margin.bottom, 4)
        self.assertEqual(margin.left, 4)
        self.assertEqual(margin.right, 4)

    def test_init_one_tuple(self):
        has_margin = HasMarginClass(margin=(4,))
        margin = has_margin.margin

        self.assertEqual(margin.top, 4)
        self.assertEqual(margin.bottom, 4)
        self.assertEqual(margin.left, 4)
        self.assertEqual(margin.right, 4)

    def test_init_two_tuple(self):
        has_margin = HasMarginClass(margin=(4, 2))
        margin = has_margin.margin

        self.assertEqual(margin.top, 2)
        self.assertEqual(margin.bottom, 2)
        self.assertEqual(margin.left, 4)
        self.assertEqual(margin.right, 4)

    def test_init_four_tuple(self):
        has_margin = HasMarginClass(margin=(4, 2, 3, 1))
        margin = has_margin.margin

        self.assertEqual(margin.top, 3)
        self.assertEqual(margin.bottom, 1)
        self.assertEqual(margin.left, 4)
        self.assertEqual(margin.right, 2)

    def test_init_margin(self):
        margin = Margin()
        has_margin = HasMarginClass(margin=margin)

        self.assertEqual(has_margin.margin, margin)

    def test_set_int(self):
        has_margin = HasMarginClass()
        with self.assertTraitChanges(has_margin, "margin", 1):
            has_margin.margin = 4

        margin = has_margin.margin
        self.assertEqual(margin.top, 4)
        self.assertEqual(margin.bottom, 4)
        self.assertEqual(margin.left, 4)
        self.assertEqual(margin.right, 4)

    def test_set_one_tuple(self):
        has_margin = HasMarginClass()
        with self.assertTraitChanges(has_margin, "margin", 1):
            has_margin.margin = (4,)

        margin = has_margin.margin

        self.assertEqual(margin.top, 4)
        self.assertEqual(margin.bottom, 4)
        self.assertEqual(margin.left, 4)
        self.assertEqual(margin.right, 4)

    def test_set_two_tuple(self):
        has_margin = HasMarginClass()
        with self.assertTraitChanges(has_margin, "margin", 1):
            has_margin.margin = (4, 2)

        margin = has_margin.margin

        self.assertEqual(margin.top, 2)
        self.assertEqual(margin.bottom, 2)
        self.assertEqual(margin.left, 4)
        self.assertEqual(margin.right, 4)

    def test_set_four_tuple(self):
        has_margin = HasMarginClass()
        with self.assertTraitChanges(has_margin, "margin", 1):
            has_margin.margin = (4, 2, 3, 1)

        margin = has_margin.margin
        self.assertEqual(margin.top, 3)
        self.assertEqual(margin.bottom, 1)
        self.assertEqual(margin.left, 4)
        self.assertEqual(margin.right, 2)

    def test_set_margin(self):
        margin = Margin()
        has_margin = HasMarginClass()
        with self.assertTraitChanges(has_margin, "margin", 1):
            has_margin.margin = margin

        self.assertEqual(has_margin.margin, margin)

    def test_set_invalid(self):
        has_margin = HasMarginClass()
        with self.assertRaises(TraitError):
            has_margin.margin = (1, 2, 3)


class TestBorder(unittest.TestCase):
    def test_defaults(self):
        border = Border()
        self.assertEqual(border.top, 0)
        self.assertEqual(border.bottom, 0)
        self.assertEqual(border.left, 0)
        self.assertEqual(border.right, 0)

    def test_init_one_arg(self):
        border = Border(4)
        self.assertEqual(border.top, 4)
        self.assertEqual(border.bottom, 4)
        self.assertEqual(border.left, 4)
        self.assertEqual(border.right, 4)

    def test_init_two_args(self):
        border = Border(4, 2)
        self.assertEqual(border.top, 2)
        self.assertEqual(border.bottom, 2)
        self.assertEqual(border.left, 4)
        self.assertEqual(border.right, 4)

    def test_init_four_args(self):
        border = Border(4, 2, 3, 1)
        self.assertEqual(border.top, 3)
        self.assertEqual(border.bottom, 1)
        self.assertEqual(border.left, 4)
        self.assertEqual(border.right, 2)


class TestHasBorder(unittest.TestCase, UnittestTools):
    def test_defaults(self):
        has_border = HasBorderClass()
        border = has_border.border

        self.assertEqual(border.top, 0)
        self.assertEqual(border.bottom, 0)
        self.assertEqual(border.left, 0)
        self.assertEqual(border.right, 0)

    def test_unspecified_default(self):
        trait = HasBorder()
        trait.default_value_type = DefaultValue.unspecified

        (dvt, dv) = trait.get_default_value()

        self.assertEqual(dvt, DefaultValue.callable_and_args)
        self.assertEqual(
            dv, (Border, (), {"top": 0, "bottom": 0, "left": 0, "right": 0})
        )

    def test_default_int(self):
        class HasBorderClass(HasTraits):

            border = HasBorder(4)

        has_border = HasBorderClass()
        border = has_border.border

        self.assertEqual(border.top, 4)
        self.assertEqual(border.bottom, 4)
        self.assertEqual(border.left, 4)
        self.assertEqual(border.right, 4)

    def test_default_one_tuple(self):
        class HasBorderClass(HasTraits):

            border = HasBorder((4,))

        has_border = HasBorderClass()
        border = has_border.border

        self.assertEqual(border.top, 4)
        self.assertEqual(border.bottom, 4)
        self.assertEqual(border.left, 4)
        self.assertEqual(border.right, 4)

    def test_default_two_tuple(self):
        class HasBorderClass(HasTraits):

            border = HasBorder((4, 2))

        has_border = HasBorderClass()
        border = has_border.border

        self.assertEqual(border.top, 2)
        self.assertEqual(border.bottom, 2)
        self.assertEqual(border.left, 4)
        self.assertEqual(border.right, 4)

    def test_default_four_tuple(self):
        class HasBorderClass(HasTraits):

            border = HasBorder((4, 2, 3, 1))

        has_border = HasBorderClass()
        border = has_border.border

        self.assertEqual(border.top, 3)
        self.assertEqual(border.bottom, 1)
        self.assertEqual(border.left, 4)
        self.assertEqual(border.right, 2)

    def test_default_border(self):
        m = Margin(left=4, right=2, top=3, bottom=1)

        class HasBorderClass(HasTraits):

            border = HasBorder(m)

        has_border = HasBorderClass()
        border = has_border.border

        self.assertEqual(border.top, 3)
        self.assertEqual(border.bottom, 1)
        self.assertEqual(border.left, 4)
        self.assertEqual(border.right, 2)

    def test_init_int(self):
        has_border = HasBorderClass(border=4)
        border = has_border.border

        self.assertEqual(border.top, 4)
        self.assertEqual(border.bottom, 4)
        self.assertEqual(border.left, 4)
        self.assertEqual(border.right, 4)

    def test_init_one_tuple(self):
        has_border = HasBorderClass(border=(4,))
        border = has_border.border

        self.assertEqual(border.top, 4)
        self.assertEqual(border.bottom, 4)
        self.assertEqual(border.left, 4)
        self.assertEqual(border.right, 4)

    def test_init_two_tuple(self):
        has_border = HasBorderClass(border=(4, 2))
        border = has_border.border

        self.assertEqual(border.top, 2)
        self.assertEqual(border.bottom, 2)
        self.assertEqual(border.left, 4)
        self.assertEqual(border.right, 4)

    def test_init_four_tuple(self):
        has_border = HasBorderClass(border=(4, 2, 3, 1))
        border = has_border.border

        self.assertEqual(border.top, 3)
        self.assertEqual(border.bottom, 1)
        self.assertEqual(border.left, 4)
        self.assertEqual(border.right, 2)

    def test_init_border(self):
        border = Border()
        has_border = HasBorderClass(border=border)

        self.assertEqual(has_border.border, border)

    def test_set_int(self):
        has_border = HasBorderClass()
        with self.assertTraitChanges(has_border, "border", 1):
            has_border.border = 4

        border = has_border.border
        self.assertEqual(border.top, 4)
        self.assertEqual(border.bottom, 4)
        self.assertEqual(border.left, 4)
        self.assertEqual(border.right, 4)

    def test_set_one_tuple(self):
        has_border = HasBorderClass()
        with self.assertTraitChanges(has_border, "border", 1):
            has_border.border = (4,)

        border = has_border.border

        self.assertEqual(border.top, 4)
        self.assertEqual(border.bottom, 4)
        self.assertEqual(border.left, 4)
        self.assertEqual(border.right, 4)

    def test_set_two_tuple(self):
        has_border = HasBorderClass()
        with self.assertTraitChanges(has_border, "border", 1):
            has_border.border = (4, 2)

        border = has_border.border

        self.assertEqual(border.top, 2)
        self.assertEqual(border.bottom, 2)
        self.assertEqual(border.left, 4)
        self.assertEqual(border.right, 4)

    def test_set_four_tuple(self):
        has_border = HasBorderClass()
        with self.assertTraitChanges(has_border, "border", 1):
            has_border.border = (4, 2, 3, 1)

        border = has_border.border
        self.assertEqual(border.top, 3)
        self.assertEqual(border.bottom, 1)
        self.assertEqual(border.left, 4)
        self.assertEqual(border.right, 2)

    def test_set_border(self):
        border = Border()
        has_border = HasBorderClass()
        with self.assertTraitChanges(has_border, "border", 1):
            has_border.border = border

        self.assertEqual(has_border.border, border)

    def test_set_invalid(self):
        has_border = HasBorderClass()
        with self.assertRaises(TraitError):
            has_border.border = (1, 2, 3)



# (C) Copyright 2005-2020 Enthought, Inc., Austin, TX
# All rights reserved.
#
# This software is provided without warranty under the terms of the BSD
# license included in LICENSE.txt and may be redistributed only under
# the conditions described in the aforementioned license. The license
# is also available online at http://www.enthought.com/licenses/BSD.txt
#
# Thanks for using Enthought open source!
# (C) Copyright 2007 Riverbank Computing Limited
# This software is provided without warranty under the terms of the BSD license.
# However, when used with the GPL version of PyQt the additional terms described in the PyQt GPL exception also apply


from pyface.qt import QtGui


from traits.api import HasTraits, Int, Property, provides, Tuple


from pyface.i_system_metrics import ISystemMetrics, MSystemMetrics


@provides(ISystemMetrics)
class SystemMetrics(MSystemMetrics, HasTraits):
    """ The toolkit specific implementation of a SystemMetrics.  See the
    ISystemMetrics interface for the API documentation.
    """

    # 'ISystemMetrics' interface -------------------------------------------

    screen_width = Property(Int)

    screen_height = Property(Int)

    dialog_background_color = Property(Tuple)

    # ------------------------------------------------------------------------
    # Private interface.
    # ------------------------------------------------------------------------

    def _get_screen_width(self):
        return QtGui.QApplication.instance().desktop().screenGeometry().width()

    def _get_screen_height(self):
        return (
            QtGui.QApplication.instance().desktop().screenGeometry().height()
        )

    def _get_dialog_background_color(self):
        color = (
            QtGui.QApplication.instance()
            .palette()
            .color(QtGui.QPalette.Window)
        )

        return (color.redF(), color.greenF(), color.blueF())



# (C) Copyright 2005-2020 Enthought, Inc., Austin, TX
# All rights reserved.
#
# This software is provided without warranty under the terms of the BSD
# license included in LICENSE.txt and may be redistributed only under
# the conditions described in the aforementioned license. The license
# is also available online at http://www.enthought.com/licenses/BSD.txt
#
# Thanks for using Enthought open source!

from itertools import combinations
import logging


from traits.api import Any, HasTraits


from .dock_pane import AREA_MAP, INVERSE_AREA_MAP
from pyface.tasks.task_layout import (
    LayoutContainer,
    PaneItem,
    Tabbed,
    Splitter,
    HSplitter,
    VSplitter,
)

# row/col orientation for AUI
ORIENTATION_NEEDS_NEW_ROW = {
    "horizontal": {"top": False, "bottom": False, "left": True, "right": True},
    "vertical": {"top": True, "bottom": True, "left": False, "right": False},
}


# Logging.
logger = logging.getLogger(__name__)


class MainWindowLayout(HasTraits):
    """ A class for applying declarative layouts to an AUI managed window.
    """

    # ------------------------------------------------------------------------
    # 'MainWindowLayout' interface.
    # ------------------------------------------------------------------------

    def get_layout(self, layout, window):
        """ Get the layout by adding sublayouts to the specified DockLayout.
        """
        logger.debug("get_layout: %s" % layout)
        layout.perspective = window._aui_manager.SavePerspective()
        logger.debug("get_layout: saving perspective %s" % layout.perspective)

    def set_layout(self, layout, window):
        """ Applies a DockLayout to the window.
        """
        logger.debug("set_layout: %s" % layout)

        if hasattr(layout, "perspective"):
            self._set_layout_from_aui(layout, window)
            return

        # Perform the layout. This will assign fixed sizes to the dock widgets
        # to enforce size constraints specified in the PaneItems.
        for name, direction in AREA_MAP.items():
            sublayout = getattr(layout, name)
            if sublayout:
                self.set_layout_for_area(sublayout, direction)

        self._add_dock_panes(window)

    def _add_dock_panes(self, window):
        # Add all panes not assigned an area by the TaskLayout.
        mgr = window._aui_manager
        for dock_pane in self.state.dock_panes:
            info = mgr.GetPane(dock_pane.pane_name)
            if not info.IsOk():
                logger.debug(
                    "_add_dock_panes: managing pane %s" % dock_pane.pane_name
                )
                dock_pane.add_to_manager()
            else:
                logger.debug(
                    "_add_dock_panes: arleady managed pane: %s"
                    % dock_pane.pane_name
                )

    def _set_layout_from_aui(self, layout, window):
        # The central pane will have already been added, but we need to add all
        # of the dock panes to the manager before the call to LoadPerspective
        logger.debug("_set_layout_from_aui: using saved perspective")
        self._add_dock_panes(window)
        logger.debug(
            "_set_layout_from_aui: restoring perspective %s"
            % layout.perspective
        )
        window._aui_manager.LoadPerspective(layout.perspective)
        for dock_pane in self.state.dock_panes:
            logger.debug("validating dock pane traits for %s" % dock_pane.id)
            dock_pane.validate_traits_from_pane_info()

    def set_layout_for_area(self, layout, direction, row=None, pos=None):
        """ Applies a LayoutItem to the specified dock area.
        """
        # AUI doesn't have full, arbitrary row/col positions, nor infinitely
        # splittable areas.  Top and bottom docks are only splittable
        # vertically, and within each vertical split each can be split
        # horizontally and that's it.  Similarly, left and right docks can
        # only be split horizontally and within each horizontal split can be
        # split vertically.
        logger.debug("set_layout_for_area: %s" % INVERSE_AREA_MAP[direction])

        if isinstance(layout, PaneItem):
            dock_pane = self._get_dock_pane(layout)
            if dock_pane is None:
                raise MainWindowLayoutError("Unknown dock pane %r" % layout)
            dock_pane.dock_area = INVERSE_AREA_MAP[direction]
            logger.debug("layout size (%d,%d)" % (layout.width, layout.height))
            dock_pane.add_to_manager(row=row, pos=pos)
            dock_pane.visible = True

        elif isinstance(layout, Tabbed):
            active_pane = first_pane = None
            for item in layout.items:
                dock_pane = self._get_dock_pane(item)
                dock_pane.dock_area = INVERSE_AREA_MAP[direction]
                if item.id == layout.active_tab:
                    active_pane = dock_pane
                dock_pane.add_to_manager(tabify_pane=first_pane)
                if not first_pane:
                    first_pane = dock_pane
                dock_pane.visible = True

            # Activate the appropriate tab, if possible.
            if not active_pane:
                # By default, AUI will activate the last widget.
                active_pane = first_pane
            if active_pane:
                mgr = active_pane.task.window._aui_manager
                info = active_pane.get_pane_info()
                mgr.ShowPane(info.window, True)

        elif isinstance(layout, Splitter):
            dock_area = INVERSE_AREA_MAP[direction]
            needs_new_row = ORIENTATION_NEEDS_NEW_ROW[layout.orientation][
                dock_area
            ]
            if needs_new_row:
                if row is None:
                    row = 0
                else:
                    row += 1
                for i, item in enumerate(layout.items):
                    self.set_layout_for_area(item, direction, row, pos)
                    row += 1
            else:
                pos = 0
                for i, item in enumerate(layout.items):
                    self.set_layout_for_area(item, direction, row, pos)
                    pos += 1

        else:
            raise MainWindowLayoutError("Unknown layout item %r" % layout)

    # ------------------------------------------------------------------------
    # 'MainWindowLayout' abstract interface.
    # ------------------------------------------------------------------------

    def _get_dock_widget(self, pane):
        """ Returns the QDockWidget associated with a PaneItem.
        """
        raise NotImplementedError

    def _get_pane(self, dock_widget):
        """ Returns a PaneItem for a QDockWidget.
        """
        raise NotImplementedError

    def _get_dock_pane(self, pane):
        """ Returns the DockPane associated with a PaneItem.
        """
        for dock_pane in self.state.dock_panes:
            if dock_pane.id == pane.id:
                return dock_pane
        return None


class MainWindowLayoutError(ValueError):
    """ Exception raised when a malformed LayoutItem is passed to the
    MainWindowLayout.
    """

    pass



# (C) Copyright 2005-2020 Enthought, Inc., Austin, TX
# All rights reserved.
#
# This software is provided without warranty under the terms of the BSD
# license included in LICENSE.txt and may be redistributed only under
# the conditions described in the aforementioned license. The license
# is also available online at http://www.enthought.com/licenses/BSD.txt
#
# Thanks for using Enthought open source!
""" A memento for a workbench window. """


from traits.api import Any, Dict, HasTraits, Str, Tuple


class WorkbenchWindowMemento(HasTraits):
    """ A memento for a workbench window. """

    # The Id of the active perspective.
    active_perspective_id = Str()

    # The memento for the editor area.
    editor_area_memento = Any()

    # Mementos for each perspective that has been seen.
    #
    # The keys are the perspective Ids, the values are the toolkit-specific
    # mementos.
    perspective_mementos = Dict(Str, Any)

    # The position of the window.
    position = Tuple()

    # The size of the window.
    size = Tuple()

    # Any extra data the toolkit implementation may want to keep.
    toolkit_data = Any()



# (C) Copyright 2005-2020 Enthought, Inc., Austin, TX
# All rights reserved.
#
# This software is provided without warranty under the terms of the BSD
# license included in LICENSE.txt and may be redistributed only under
# the conditions described in the aforementioned license. The license
# is also available online at http://www.enthought.com/licenses/BSD.txt
#
# Thanks for using Enthought open source!
""" Define a base Task application class to create the event loop, and launch
the creation of tasks and corresponding windows.
"""


from functools import partial
import logging

from traits.api import (
    Callable,
    HasStrictTraits,
    List,
    Instance,
    Property,
    Str,
    cached_property,
    on_trait_change,
)

from pyface.gui_application import GUIApplication

logger = logging.getLogger(__name__)


class TaskFactory(HasStrictTraits):
    """ A factory for creating a Task with some additional metadata.
    """

    #: The task factory's unique identifier. This ID is assigned to all tasks
    #: created by the factory.
    id = Str()

    #: The task factory's user-visible name.
    name = Str()

    #: A callable with the following signature:
    #:
    #:     callable(**traits) -> Task
    #:
    #: Often this attribute will simply be a Task subclass.
    factory = Callable

    def create(self, **traits):
        """ Creates the Task.

        The default implementation simply calls the 'factory' attribute.
        """
        return self.factory(**traits)


class TasksApplication(GUIApplication):
    """ A base class for Pyface tasks applications.
    """

    # -------------------------------------------------------------------------
    # 'TaskApplication' interface
    # -------------------------------------------------------------------------

    # Task management --------------------------------------------------------

    #: List of all running tasks
    tasks = List(Instance("pyface.tasks.task.Task"))

    #: Currently active Task if any.
    active_task = Property(depends_on="active_window.active_task")

    #: List of all application task factories.
    task_factories = List()

    #: The default layout for the application. If not specified, a single
    #: window will be created with the first available task factory.
    default_layout = List(
        Instance("pyface.tasks.task_window_layout.TaskWindowLayout")
    )

    #: Hook to add global schema additions to tasks/windows
    extra_actions = List(
        Instance("pyface.tasks.action.schema_addition.SchemaAddition")
    )

    #: Hook to add global dock pane additions to tasks/windows
    extra_dock_pane_factories = List(Callable)

    # Window lifecycle methods -----------------------------------------------

    def create_task(self, id):
        """ Creates the Task with the specified ID.

        Parameters
        ----------
        id : str
            The id of the task factory to use.

        Returns
        -------
        The new Task, or None if there is not a suitable TaskFactory.
        """
        factory = self._get_task_factory(id)
        if factory is None:
            logger.warning("Could not find task factory {}".format(id))
            return None

        task = factory.create(id=factory.id)
        task.extra_actions.extend(self.extra_actions)
        task.extra_dock_pane_factories.extend(self.extra_dock_pane_factories)
        return task

    def create_window(self, layout=None, **kwargs):
        """ Connect task to application and open task in a new window.

        Parameters
        ----------
        layout : TaskLayout instance or None
            The pane layout for the window.
        **kwargs : dict
            Additional keyword arguments to pass to the window factory.


        Returns
        -------
        window : ITaskWindow instance or None
            The new TaskWindow.
        """
        from pyface.tasks.task_window_layout import TaskWindowLayout

        window = super(TasksApplication, self).create_window(**kwargs)

        if layout is not None:
            for task_id in layout.get_tasks():
                task = self.create_task(task_id)
                if task is not None:
                    window.add_task(task)
                else:
                    msg = "Missing factory for task with ID %r"
                    logger.error(msg, task_id)
        else:
            # Create an empty layout to set default size and position only
            layout = TaskWindowLayout()

        window.set_window_layout(layout)

        return window

    def _create_windows(self):
        """ Create the initial windows to display from the default layout.
        """
        for layout in self.default_layout:
            window = self.create_window(layout)
            self.add_window(window)
            self.active_window = window

        # -------------------------------------------------------------------------
        # Private interface
        # -------------------------------------------------------------------------

    def _get_task_factory(self, id):
        """ Returns the TaskFactory with the specified ID, or None.
        """
        for factory in self.task_factories:
            if factory.id == id:
                return factory
        return None

    # Destruction utilities ---------------------------------------------------

    @on_trait_change("windows:closed")
    def _on_window_closed(self, window, trait, old, new):
        """ Listener that ensures window handles are released when closed.
        """
        if getattr(window, "active_task", None) in self.tasks:
            self.tasks.remove(window.active_task)
        super(TasksApplication, self)._on_window_closed(
            window, trait, old, new
        )

    # Trait initializers and property getters ---------------------------------

    def _window_factory_default(self):
        """ Default to TaskWindow.

        This will be sufficient in many cases as customized behaviour comes
        from the Task and the TaskWindow is just a shell.
        """
        from pyface.tasks.task_window import TaskWindow

        return TaskWindow

    def _default_layout_default(self):
        from pyface.tasks.task_window_layout import TaskWindowLayout

        window_layout = TaskWindowLayout()
        if self.task_factories:
            window_layout.items = [self.task_factories[0].id]
        return [window_layout]

    def _extra_actions_default(self):
        """ Extra application-wide menu items

        This adds a collection of standard Tasks application menu items and
        groups to a Task's set of menus.  Whether or not they actually appear
        depends on whether the appropriate menus are provided by the Task.

        These default additions assume that the window will hold an editor pane
        so that Ctrl-N and Ctrl-W will be bound to creating/closing new editors
        rather than new task windows.
        """
        from pyface.action.api import (
            AboutAction,
            CloseActiveWindowAction,
            ExitAction,
        )
        from pyface.tasks.action.api import (
            CreateTaskWindowAction,
            SchemaAddition,
            SMenu,
            TaskWindowToggleGroup,
        )

        return [
            SchemaAddition(
                factory=CreateTaskWindowAction.factory(
                    application=self, accelerator="Ctrl+Shift+N"
                ),
                path="MenuBar/File/new_group",
            ),
            SchemaAddition(
                id="close_action",
                factory=CloseActiveWindowAction.factory(
                    application=self, accelerator="Ctrl+Shift+W"
                ),
                path="MenuBar/File/close_group",
            ),
            SchemaAddition(
                id="exit_action",
                factory=ExitAction.factory(application=self),
                path="MenuBar/File/close_group",
                absolute_position="last",
            ),
            SchemaAddition(
                # id='Window',
                factory=lambda: SMenu(
                    TaskWindowToggleGroup(application=self),
                    id="Window",
                    name="&Window",
                ),
                path="MenuBar",
                after="View",
                before="Help",
            ),
            SchemaAddition(
                id="about_action",
                factory=AboutAction.factory(application=self),
                path="MenuBar/Help",
                absolute_position="first",
            ),
        ]

    @cached_property
    def _get_active_task(self):
        if self.active_window is not None:
            return getattr(self.active_window, "active_task", None)
        else:
            return None



# (C) Copyright 2005-2020 Enthought, Inc., Austin, TX
# All rights reserved.
#
# This software is provided without warranty under the terms of the BSD
# license included in LICENSE.txt and may be redistributed only under
# the conditions described in the aforementioned license. The license
# is also available online at http://www.enthought.com/licenses/BSD.txt
#
# Thanks for using Enthought open source!

""" Classes to provide a switcher. """


import wx
from wx.lib.scrolledpanel import ScrolledPanel as wxScrolledPanel


from traits.api import HasTraits, Int


class SwitcherModel(HasTraits):
    """ Base class for switcher models. """

    # The index of the selected 'page'.
    selected = Int(-1)

    def __init__(self):
        """ Creates a new switcher model. """

        # The items to display in the switcher control.
        self.items = []  # (str label, object value)

        return

    # ------------------------------------------------------------------------
    # 'SwitcherModel' interface.
    # ------------------------------------------------------------------------

    def create_page(self, parent, index):
        """ Creates a page for the switcher panel. """

        raise NotImplementedError


class SwitcherControl(wx.Panel):
    """ The default switcher control (a combo box). """

    def __init__(self, parent, id, model, label=None, **kw):
        """ Creates a new switcher control. """

        # Base-class constructor.
        wx.Panel.__init__(self, parent, id, **kw)

        # The switcher model that we are a controller for.
        self.model = model

        # The optional label.
        self.label = label

        # Create the widget!
        self._create_widget(model, label)

        # Listen for when the selected item in the model is changed.
        model.on_trait_change(self._on_selected_changed, "selected")

        return

    # ------------------------------------------------------------------------
    # Trait event handlers.
    # ------------------------------------------------------------------------

    def _on_selected_changed(self, selected):
        """ Called when the selected item in the model is changed. """

        self.combo.SetSelection(selected)

        return

    # ------------------------------------------------------------------------
    # wx event handlers.
    # ------------------------------------------------------------------------

    def _on_combobox(self, event):
        """ Called when the combo box selection is changed. """

        combo = event.GetEventObject()

        # Update the model.
        self.model.selected = combo.GetSelection()

        return

    # ------------------------------------------------------------------------
    # Private interface.
    # ------------------------------------------------------------------------

    def _create_widget(self, model, label):
        """ Creates the widget."""

        self.sizer = sizer = wx.BoxSizer(wx.VERTICAL)
        self.SetSizer(sizer)
        self.SetAutoLayout(True)
        ##self.SetBackgroundColour("light grey")

        # Switcher combo.
        sizer.Add(self._combo(self, model, label), 1, wx.EXPAND)

        # Resize the panel to match the sizer's minimal size.
        sizer.Fit(self)

    def _combo(self, parent, model, label):
        """ Creates the switcher combo. """

        sizer = wx.BoxSizer(wx.HORIZONTAL)

        # Label.
        if label is not None:
            text = wx.StaticText(parent, -1, label)
            sizer.Add(text, 0, wx.ALIGN_CENTER | wx.ALL, 5)

        # Combo.
        self.combo = combo = wx.ComboBox(
            parent, -1, style=wx.CB_DROPDOWN | wx.CB_READONLY
        )
        sizer.Add(combo, 1, wx.EXPAND | wx.ALIGN_CENTER | wx.ALL, 5)

        # Ask the model for the available options.
        items = model.items
        if len(items) > 0:
            for name, data in model.items:
                combo.Append(name, data)

        # Listen for changes to the selected item.
        self.Bind(wx.EVT_COMBOBOX, self._on_combobox, id=combo.GetId())

        # If the model's selected variable has been set ...
        if model.selected != -1:
            combo.SetSelection(model.selected)

        return sizer


class SwitcherPanel(wxScrolledPanel):
    """ The default switcher panel. """

    def __init__(self, parent, id, model, label=None, cache=True, **kw):

        # Base-class constructor.
        wxScrolledPanel.__init__(self, parent, id, **kw)
        self.SetupScrolling()

        # The switcher model that we are a panel for.
        self.model = model

        # Should we cache pages as we create them?
        self.cache = cache

        # The page cache (if caching was requested).
        self._page_cache = {}

        # The currently displayed page.
        self.current = None

        # Create the widget!
        self._create_widget(model, label)

        # Listen for when the selected item in the model is changed.
        # model.on_trait_change(self._on_selected_changed, 'selected')

        return

    # ------------------------------------------------------------------------
    # 'SwitcherPanel' interface.
    # ------------------------------------------------------------------------

    def show_page(self, index):
        """ Shows the page at the specified index. """

        self._show_page(index)

        return

    # ------------------------------------------------------------------------
    # Trait event handlers.
    # ------------------------------------------------------------------------

    def _on_selected_changed(self, selected):
        """ Called when the selected item in the model is changed. """

        self._show_page(selected)

        return

    # ------------------------------------------------------------------------
    # Private interface.
    # ------------------------------------------------------------------------

    def _create_widget(self, model, label):
        """ Creates the widget. """

        self.sizer = sizer = wx.BoxSizer(wx.VERTICAL)
        self.SetSizer(sizer)
        self.SetAutoLayout(True)
        ##self.SetBackgroundColour('red')

        # if model.selected != -1:
        #    self._show_page(model.selected)

        # Nothing to add here as we add the panel contents lazily!
        pass

        # Resize the panel to match the sizer's minimal size.
        sizer.Fit(self)

    def _show_page(self, index):
        """ Shows the page at the specified index. """

        # If a page is already displayed then hide it.
        if self.current is not None:
            current_size = self.current.GetSize()
            self.current.Show(False)
            self.sizer.Remove(self.current)

        # Is the page in the cache?
        page = self._page_cache.get(index)
        if not self.cache or page is None:
            # If not then ask our panel factory to create it.
            page = self.model.create_page(self, index)

            # Add it to the cache!
            self._page_cache[index] = page

        # if self.current is not None:
        #    page.SetSize(current_size)

        # Display the page.
        self.sizer.Add(page, 15, wx.EXPAND, 5)
        page.Show(True)

        self.current = page

        # Force a new layout of the sizer's children but KEEPING the current
        # dimension.
        self.sizer.Layout()
        # self.sizer.Fit(self)
        # self.SetupScrolling()


class Switcher(wx.Panel):
    """ A switcher. """

    def __init__(self, parent, id, model, label=None, **kw):
        """ Create a new switcher. """

        # Base-class constructor.
        wx.Panel.__init__(self, parent, id, **kw)

        # The model that we are a switcher for.
        self.model = model

        # Create the widget!
        self._create_widget(model, label)

        return

    # ------------------------------------------------------------------------
    # Private interface.
    # ------------------------------------------------------------------------

    def _create_widget(self, model, label):
        """ Creates the widget. """

        self.sizer = sizer = wx.BoxSizer(wx.VERTICAL)
        self.SetSizer(sizer)
        self.SetAutoLayout(True)

        # Switcher control.
        self.control = control = SwitcherControl(self, -1, model, label)
        sizer.Add(control, 0, wx.EXPAND)

        # Switcher panel.
        self.panel = panel = SwitcherPanel(self, -1, model, label)
        sizer.Add(panel, 1, wx.EXPAND)

        # Resize the panel to match the sizer's minimal size.
        sizer.Fit(self)

        return



# (C) Copyright 2005-2020 Enthought, Inc., Austin, TX
# All rights reserved.
#
# This software is provided without warranty under the terms of the BSD
# license included in LICENSE.txt and may be redistributed only under
# the conditions described in the aforementioned license. The license
# is also available online at http://www.enthought.com/licenses/BSD.txt
#
# Thanks for using Enthought open source!

import wx
from wx import stc

import keyword

# ----------------------------------------------------------------------

demoText = """\
## This version of the editor has been set up to edit Python source
## code.  Here is a copy of wxPython/demo/Main.py to play with.


"""

# ----------------------------------------------------------------------


if wx.Platform == "__WXMSW__":
    faces = {
        "times": "Times New Roman",
        "mono": "Courier New",
        "helv": "Arial",
        "other": "Comic Sans MS",
        "size": 10,
        "size2": 8,
    }
else:
    faces = {
        "times": "Times",
        "mono": "Courier",
        "helv": "Helvetica",
        "other": "new century schoolbook",
        "size": 12,
        "size2": 10,
    }


# ----------------------------------------------------------------------


class PythonSTC(stc.StyledTextCtrl):
    def __init__(self, parent, ID):
        stc.StyledTextCtrl.__init__(
            self, parent, ID, style=wx.NO_FULL_REPAINT_ON_RESIZE
        )

        self.CmdKeyAssign(ord("B"), stc.STC_SCMOD_CTRL, stc.STC_CMD_ZOOMIN)
        self.CmdKeyAssign(ord("N"), stc.STC_SCMOD_CTRL, stc.STC_CMD_ZOOMOUT)

        self.SetLexer(stc.STC_LEX_PYTHON)
        self.SetKeyWords(0, " ".join(keyword.kwlist))

        self.SetProperty("fold", "1")
        self.SetProperty("tab.timmy.whinge.level", "1")
        self.SetMargins(0, 0)

        self.SetViewWhiteSpace(False)
        # self.SetBufferedDraw(False)

        self.SetEdgeMode(stc.STC_EDGE_BACKGROUND)
        self.SetEdgeColumn(78)

        # Setup a margin to hold fold markers
        #  WHAT IS THIS VALUE?  WHAT ARE THE OTHER FLAGS?  DOES IT MATTER?
        self.SetFoldFlags(16)
        # mic
        # self.SetMarginType(2, stc.STC_MARGIN_SYMBOL)
        # self.SetMarginMask(2, stc.STC_MASK_FOLDERS)
        # self.SetMarginSensitive(2, True)
        # self.SetMarginWidth(2, 12)
        # line numbers in the margin
        self.SetMarginType(1, stc.STC_MARGIN_NUMBER)
        self.SetMarginWidth(1, 25)

        if 0:  # simple folder marks, like the old version
            self.MarkerDefine(
                stc.STC_MARKNUM_FOLDER, stc.STC_MARK_ARROW, "navy", "navy"
            )
            self.MarkerDefine(
                stc.STC_MARKNUM_FOLDEROPEN,
                stc.STC_MARK_ARROWDOWN,
                "navy",
                "navy",
            )
            # Set these to an invisible mark
            self.MarkerDefine(
                stc.STC_MARKNUM_FOLDEROPENMID,
                stc.STC_MARK_BACKGROUND,
                "white",
                "black",
            )
            self.MarkerDefine(
                stc.STC_MARKNUM_FOLDERMIDTAIL,
                stc.STC_MARK_BACKGROUND,
                "white",
                "black",
            )
            self.MarkerDefine(
                stc.STC_MARKNUM_FOLDERSUB,
                stc.STC_MARK_BACKGROUND,
                "white",
                "black",
            )
            self.MarkerDefine(
                stc.STC_MARKNUM_FOLDERTAIL,
                stc.STC_MARK_BACKGROUND,
                "white",
                "black",
            )

        elif 0:  # more involved "outlining" folder marks
            self.MarkerDefine(
                stc.STC_MARKNUM_FOLDEREND,
                stc.STC_MARK_BOXPLUSCONNECTED,
                "white",
                "black",
            )
            self.MarkerDefine(
                stc.STC_MARKNUM_FOLDEROPENMID,
                stc.STC_MARK_BOXMINUSCONNECTED,
                "white",
                "black",
            )
            self.MarkerDefine(
                stc.STC_MARKNUM_FOLDERMIDTAIL,
                stc.STC_MARK_TCORNER,
                "white",
                "black",
            )
            self.MarkerDefine(
                stc.STC_MARKNUM_FOLDERTAIL,
                stc.STC_MARK_LCORNER,
                "white",
                "black",
            )
            self.MarkerDefine(
                stc.STC_MARKNUM_FOLDERSUB, stc.STC_MARK_VLINE, "white", "black"
            )
            self.MarkerDefine(
                stc.STC_MARKNUM_FOLDER, stc.STC_MARK_BOXPLUS, "white", "black"
            )
            self.MarkerDefine(
                stc.STC_MARKNUM_FOLDEROPEN,
                stc.STC_MARK_BOXMINUS,
                "white",
                "black",
            )

        self.Bind(stc.EVT_STC_UPDATEUI, self.OnUpdateUI)
        self.Bind(stc.EVT_STC_MARGINCLICK, self.OnMarginClick)

        # Make some styles,  The lexer defines what each style is used for, we
        # just have to define what each style looks like.  This set is adapted
        # from Scintilla sample property files.

        self.StyleClearAll()

        # Global default styles for all languages
        self.StyleSetSpec(
            stc.STC_STYLE_DEFAULT, "face:%(helv)s,size:%(size)d" % faces
        )
        self.StyleSetSpec(
            stc.STC_STYLE_LINENUMBER,
            "back:#C0C0C0,face:%(helv)s,size:%(size2)d" % faces,
        )
        self.StyleSetSpec(stc.STC_STYLE_CONTROLCHAR, "face:%(other)s" % faces)
        self.StyleSetSpec(
            stc.STC_STYLE_BRACELIGHT, "fore:#FFFFFF,back:#0000FF,bold"
        )
        self.StyleSetSpec(
            stc.STC_STYLE_BRACEBAD, "fore:#000000,back:#FF0000,bold"
        )

        # Python styles
        # White space
        self.StyleSetSpec(
            stc.STC_P_DEFAULT,
            "fore:#808080,face:%(helv)s,size:%(size)d" % faces,
        )
        # Comment
        self.StyleSetSpec(
            stc.STC_P_COMMENTLINE,
            "fore:#007F00,face:%(other)s,size:%(size)d" % faces,
        )
        # Number
        self.StyleSetSpec(
            stc.STC_P_NUMBER, "fore:#007F7F,size:%(size)d" % faces
        )
        # String
        self.StyleSetSpec(
            stc.STC_P_STRING,
            "fore:#7F007F,italic,face:%(times)s,size:%(size)d" % faces,
        )
        # Single quoted string
        self.StyleSetSpec(
            stc.STC_P_CHARACTER,
            "fore:#7F007F,italic,face:%(times)s,size:%(size)d" % faces,
        )
        # Keyword
        self.StyleSetSpec(
            stc.STC_P_WORD, "fore:#00007F,bold,size:%(size)d" % faces
        )
        # Triple quotes
        self.StyleSetSpec(
            stc.STC_P_TRIPLE, "fore:#7F0000,size:%(size)d" % faces
        )
        # Triple double quotes
        self.StyleSetSpec(
            stc.STC_P_TRIPLEDOUBLE, "fore:#7F0000,size:%(size)d" % faces
        )
        # Class name definition
        self.StyleSetSpec(
            stc.STC_P_CLASSNAME,
            "fore:#0000FF,bold,underline,size:%(size)d" % faces,
        )
        # Function or method name definition
        self.StyleSetSpec(
            stc.STC_P_DEFNAME, "fore:#007F7F,bold,size:%(size)d" % faces
        )
        # Operators
        self.StyleSetSpec(stc.STC_P_OPERATOR, "bold,size:%(size)d" % faces)
        # Identifiers
        self.StyleSetSpec(
            stc.STC_P_IDENTIFIER,
            "fore:#808080,face:%(helv)s,size:%(size)d" % faces,
        )
        # Comment-blocks
        self.StyleSetSpec(
            stc.STC_P_COMMENTBLOCK, "fore:#7F7F7F,size:%(size)d" % faces
        )
        # End of line where string is not closed
        self.StyleSetSpec(
            stc.STC_P_STRINGEOL,
            "fore:#000000,face:%(mono)s,back:#E0C0E0,eol,size:%(size)d"
            % faces,
        )

        self.SetCaretForeground("BLUE")

        self.Bind(wx.EVT_KEY_DOWN, self.OnKeyPressed)

    def OnKeyPressed(self, event):
        if self.CallTipActive():
            self.CallTipCancel()

        key = event.KeyCode

        if key == 32 and event.ControlDown():
            pos = self.GetCurrentPos()
            # Tips
            if event.ShiftDown():
                self.CallTipSetBackground("yellow")
                self.CallTipShow(pos, "param1, param2")
            # Code completion
            else:
                # fixme: What is this mess!!!

                # lst = []
                # for x in range(50000):
                #    lst.append('%05d' % x)
                # st = " ".join(lst)
                # print len(st)
                # self.AutoCompShow(0, st)

                # fixme: What is this mess!!!
                kw = keyword.kwlist[:]
                kw.append("zzzzzz")
                kw.append("aaaaa")
                kw.append("__init__")
                kw.append("zzaaaaa")
                kw.append("zzbaaaa")
                kw.append("this_is_a_longer_value")
                kw.append("this_is_a_much_much_much_much_longer_value")

                kw.sort()  # Python sorts are case sensitive
                self.AutoCompSetIgnoreCase(False)  # so this needs to match

                self.AutoCompShow(0, " ".join(kw))
        else:
            event.Skip()

    def OnUpdateUI(self, evt):
        # check for matching braces
        braceAtCaret = -1
        braceOpposite = -1
        charBefore = None
        caretPos = self.GetCurrentPos()
        if caretPos > 0:
            charBefore = self.GetCharAt(caretPos - 1)
            styleBefore = self.GetStyleAt(caretPos - 1)

        # check before
        if (
            charBefore
            and chr(charBefore) in "[]{}()"
            and styleBefore == stc.STC_P_OPERATOR
        ):
            braceAtCaret = caretPos - 1

        # check after
        if braceAtCaret < 0:
            charAfter = self.GetCharAt(caretPos)
            styleAfter = self.GetStyleAt(caretPos)
            if (
                charAfter
                and chr(charAfter) in "[]{}()"
                and styleAfter == stc.STC_P_OPERATOR
            ):
                braceAtCaret = caretPos

        if braceAtCaret >= 0:
            braceOpposite = self.BraceMatch(braceAtCaret)

        if braceAtCaret != -1 and braceOpposite == -1:
            self.BraceBadLight(braceAtCaret)
        else:
            self.BraceHighlight(braceAtCaret, braceOpposite)
            # pt = self.PointFromPosition(braceOpposite)
            # self.Refresh(True, Rect(pt.x, pt.y, 5,5))
            # print pt
            # self.Refresh(False)

    def OnMarginClick(self, evt):
        # fold and unfold as needed
        if evt.GetMargin() == 2:
            if evt.GetShift() and evt.GetControl():
                self.FoldAll()
            else:
                lineClicked = self.LineFromPosition(evt.GetPosition())
                if (
                    self.GetFoldLevel(lineClicked)
                    & stc.STC_FOLDLEVELHEADERFLAG
                ):
                    if evt.GetShift():
                        self.SetFoldExpanded(lineClicked, True)
                        self.Expand(lineClicked, True, True, 1)
                    elif evt.GetControl():
                        if self.GetFoldExpanded(lineClicked):
                            self.SetFoldExpanded(lineClicked, False)
                            self.Expand(lineClicked, False, True, 0)
                        else:
                            self.SetFoldExpanded(lineClicked, True)
                            self.Expand(lineClicked, True, True, 100)
                    else:
                        self.ToggleFold(lineClicked)

    def FoldAll(self):
        lineCount = self.GetLineCount()
        expanding = True

        # find out if we are folding or unfolding
        for lineNum in range(lineCount):
            if self.GetFoldLevel(lineNum) & stc.STC_FOLDLEVELHEADERFLAG:
                expanding = not self.GetFoldExpanded(lineNum)
                break

        lineNum = 0
        while lineNum < lineCount:
            level = self.GetFoldLevel(lineNum)
            if (
                level & stc.STC_FOLDLEVELHEADERFLAG
                and (level & stc.STC_FOLDLEVELNUMBERMASK)
                == stc.STC_FOLDLEVELBASE
            ):

                if expanding:
                    self.SetFoldExpanded(lineNum, True)
                    lineNum = self.Expand(lineNum, True)
                    lineNum = lineNum - 1
                else:
                    lastChild = self.GetLastChild(lineNum, -1)
                    self.SetFoldExpanded(lineNum, False)
                    if lastChild > lineNum:
                        self.HideLines(lineNum + 1, lastChild)

            lineNum = lineNum + 1

    def Expand(self, line, doExpand, force=False, visLevels=0, level=-1):
        lastChild = self.GetLastChild(line, level)
        line = line + 1
        while line <= lastChild:
            if force:
                if visLevels > 0:
                    self.ShowLines(line, line)
                else:
                    self.HideLines(line, line)
            else:
                if doExpand:
                    self.ShowLines(line, line)

            if level == -1:
                level = self.GetFoldLevel(line)

            if level & stc.STC_FOLDLEVELHEADERFLAG:
                if force:
                    if visLevels > 1:
                        self.SetFoldExpanded(line, True)
                    else:
                        self.SetFoldExpanded(line, False)
                    line = self.Expand(line, doExpand, force, visLevels - 1)

                else:
                    if doExpand and self.GetFoldExpanded(line):
                        line = self.Expand(line, True, force, visLevels - 1)
                    else:
                        line = self.Expand(line, False, force, visLevels - 1)
            else:
                line = line + 1

        return line


# ----------------------------------------------------------------------

_USE_PANEL = 1


def runTest(frame, nb, log):
    if not _USE_PANEL:
        ed = p = stc.PythonSTC(nb, -1)
    else:
        p = wx.Panel(nb, -1, style=wx.NO_FULL_REPAINT_ON_RESIZE)
        ed = PythonSTC(p, -1)
        s = wx.BoxSizer(wx.HORIZONTAL)
        s.Add(ed, 1, wx.EXPAND)
        p.SetSizer(s)
        p.SetAutoLayout(True)

    ed.SetText(demoText + open("Main.py").read())
    ed.EmptyUndoBuffer()
    ed.Colourise(0, -1)

    # line numbers in the margin
    ed.SetMarginType(1, stc.STC_MARGIN_NUMBER)
    ed.SetMarginWidth(1, 25)

    return p


# ----------------------------------------------------------------------


overview = """\
<html><body>
Once again, no docs yet.  <b>Sorry.</b>  But <a href="data/stc.h.html">this</a>
and <a href="http://www.scintilla.org/ScintillaDoc.html">this</a> should
be helpful.
</body><html>
"""


if __name__ == "__main__":
    # fixme: This has been re-factored into not working.  No run module.
    import sys, os
    import run

    run.main(["", os.path.basename(sys.argv[0])])

# ----------------------------------------------------------------------



# (C) Copyright 2005-2020 Enthought, Inc., Austin, TX
# All rights reserved.
#
# This software is provided without warranty under the terms of the BSD
# license included in LICENSE.txt and may be redistributed only under
# the conditions described in the aforementioned license. The license
# is also available online at http://www.enthought.com/licenses/BSD.txt
#
# Thanks for using Enthought open source!

import os, wx
from traits.util.resource import get_path


def get_bitmap(root, name):
    """
    Convenience function that returns a bitmap
    root - either an instance of a class or a path
    name - name of png file to load
    """
    path = os.path.join(get_path(root), name)
    bmp = wx.Bitmap(path, wx.BITMAP_TYPE_PNG)
    return bmp



# (C) Copyright 2005-2020 Enthought, Inc., Austin, TX
# All rights reserved.
#
# This software is provided without warranty under the terms of the BSD
# license included in LICENSE.txt and may be redistributed only under
# the conditions described in the aforementioned license. The license
# is also available online at http://www.enthought.com/licenses/BSD.txt
#
# Thanks for using Enthought open source!

""" Mix-in class for split widgets. """


from traits.api import Callable, Enum, Float, Interface


class ISplitWidget(Interface):
    """ Mix-in class for split widgets.

    A split widget is one that is split in two either horizontally or
    vertically.
    """

    # 'ISplitWidget' interface ---------------------------------------------

    #: The direction in which the widget is split.
    #
    #: Splitting vertically means there will be a left hand panel and a right
    #: hand panel, splitting horizontally means there will be a top panel and
    #: a bottom panel.
    direction = Enum("vertical", "vertical", "horizontal")

    #: The ratio of the size of the left/top pane to the right/bottom pane.
    ratio = Float(0.5)

    #: An optional callable that provides the left hand/top panel.
    lhs = Callable

    #: An optional callable that provides the right hand/bottom panel.
    rhs = Callable

    # ------------------------------------------------------------------------
    # Protected 'ISplitWidget' interface.
    # ------------------------------------------------------------------------

    def _create_splitter(self, parent):
        """ Create the toolkit-specific control that represents the widget.

        Parameters
        ----------
        parent : toolkit control
            The toolkit control that contains the splitter.

        Returns
        -------
        splitter : toolkit control
            The toolkit control for the splitter.
        """

    def _create_lhs(self, parent):
        """ Creates the left hand/top panel depending on the direction.

        Parameters
        ----------
        parent : toolkit control
            The splitter's toolkit control.

        Returns
        -------
        lhs : toolkit control
            The toolkit control for the lhs.
        """

    def _create_rhs(self, parent):
        """ Creates the right hand/bottom panel depending on the direction.

        Parameters
        ----------
        parent : toolkit control
            The splitter's toolkit control.

        Returns
        -------
        rhs : toolkit control
            The toolkit control for the rhs.
        """


class MSplitWidget(object):
    """ The mixin class that contains common code for toolkit specific
    implementations of the ISplitWidget interface.
    """



# (C) Copyright 2005-2020 Enthought, Inc., Austin, TX
# All rights reserved.
#
# This software is provided without warranty under the terms of the BSD
# license included in LICENSE.txt and may be redistributed only under
# the conditions described in the aforementioned license. The license
# is also available online at http://www.enthought.com/licenses/BSD.txt
#
# Thanks for using Enthought open source!


from traits.api import Any, HasTraits


class GridCellRenderer(HasTraits):

    # The toolkit-specific renderer for this cell.
    renderer = Any()

    # A handler to be invoked on right-button mouse clicks.
    def on_right_click(self, grid, row, col):
        pass

    # A handler to be invoked on right-button mouse double clicks.
    def on_right_dclick(self, grid, row, col):
        pass

    # A handler to be invoked on left-button mouse clicks.
    def on_left_click(self, grid, row, col):
        pass

    # A handler to be invoked on left-button mouse double clicks.
    def on_left_dclick(self, grid, row, col):
        pass

    # A handler to be invoked on key press.
    def on_key(self, grid, row, col, key_event):
        pass

    # Clean-up!
    def dispose(self):
        pass



# (C) Copyright 2005-2020 Enthought, Inc., Austin, TX
# All rights reserved.
#
# This software is provided without warranty under the terms of the BSD
# license included in LICENSE.txt and may be redistributed only under
# the conditions described in the aforementioned license. The license
# is also available online at http://www.enthought.com/licenses/BSD.txt
#
# Thanks for using Enthought open source!


import unittest


from pyface.image_resource import ImageResource
from ..combo_field import ComboField
from .field_mixin import FieldMixin


class TestComboField(FieldMixin, unittest.TestCase):
    def _create_widget(self):
        return ComboField(
            parent=self.parent.control,
            value="one",
            values=["one", "two", "three", "four"],
            tooltip="Dummy",
        )

    # Tests ------------------------------------------------------------------

    def test_combo_field(self):
        self._create_widget_control()

        self.widget.value = "two"
        self.gui.process_events()

        self.assertEqual(self.widget._get_control_value(), "two")
        self.assertEqual(self.widget._get_control_text(), "two")

    def test_combo_field_set(self):
        self._create_widget_control()

        with self.assertTraitChanges(self.widget, "value", count=1):
            self.widget._set_control_value("two")
            self.gui.process_events()

        self.assertEqual(self.widget.value, "two")

    def test_combo_field_formatter(self):
        self.widget.formatter = str
        self.widget.values = [0, 1, 2, 3]
        self._create_widget_control()

        self.widget.value = 2
        self.gui.process_events()

        self.assertEqual(self.widget._get_control_value(), 2)
        self.assertEqual(self.widget._get_control_text(), "2")

    def test_combo_field_formatter_changed(self):
        self.widget.values = [1, 2, 3, 4]
        self.widget.value = 2
        self.widget.formatter = str
        self._create_widget_control()

        self.widget.formatter = "Number {}".format
        self.gui.process_events()

        self.assertEqual(self.widget._get_control_value(), 2)
        self.assertEqual(self.widget._get_control_text(), "Number 2")

    def test_combo_field_formatter_set(self):
        self.widget.values = [1, 2, 3, 4]
        self.widget.formatter = str
        self._create_widget_control()

        with self.assertTraitChanges(self.widget, "value", count=1):
            self.widget._set_control_value(2)
            self.gui.process_events()

        self.assertEqual(self.widget.value, 2)

    def test_combo_field_icon_formatter(self):
        image = ImageResource("question")
        self.widget.values = [1, 2, 3, 4]
        self.widget.formatter = lambda x: (image, str(x))
        self._create_widget_control()

        self.widget.value = 2
        self.gui.process_events()

        self.assertEqual(self.widget._get_control_value(), 2)
        self.assertEqual(self.widget._get_control_text(), "2")

    def test_combo_field_values(self):
        self._create_widget_control()

        self.widget.values = ["four", "five", "one", "six"]
        self.gui.process_events()

        # XXX different results in Wx and Qt
        # As best I can tell, difference is at the Traits level
        # On Qt setting 'values' sets 'value' to "four" before combofield
        # handler sees it.  On Wx it remains  as "one" at point of handler
        # call.  Possibly down to dictionary ordering or something
        # similar.
        self.assertIn(self.widget.value, {"one", "four"})



# (C) Copyright 2005-2020 Enthought, Inc., Austin, TX
# All rights reserved.
#
# This software is provided without warranty under the terms of the BSD
# license included in LICENSE.txt and may be redistributed only under
# the conditions described in the aforementioned license. The license
# is also available online at http://www.enthought.com/licenses/BSD.txt
#
# Thanks for using Enthought open source!

""" Abstract base class for all viewers. """


from pyface.widget import Widget


class Viewer(Widget):
    """ Abstract base class for all viewers.

    A viewer is a model-based adapter on some underlying toolkit-specific
    widget.

    """

    pass



# (C) Copyright 2005-2020 Enthought, Inc., Austin, TX
# All rights reserved.
#
# This software is provided without warranty under the terms of the BSD
# license included in LICENSE.txt and may be redistributed only under
# the conditions described in the aforementioned license. The license
# is also available online at http://www.enthought.com/licenses/BSD.txt
#
# Thanks for using Enthought open source!

""" A Layered panel. """


import wx

from traits.api import Instance


from .expandable_header import ExpandableHeader
from .image_resource import ImageResource
from .widget import Widget


class ExpandablePanel(Widget):
    """ An expandable panel. """

    # The default style.
    STYLE = wx.CLIP_CHILDREN

    collapsed_image = Instance(ImageResource, ImageResource("mycarat1"))
    expanded_image = Instance(ImageResource, ImageResource("mycarat2"))

    # ------------------------------------------------------------------------
    # 'object' interface.
    # ------------------------------------------------------------------------

    def __init__(self, parent, **traits):
        """ Creates a new LayeredPanel. """

        # Base class constructor.
        super(ExpandablePanel, self).__init__(**traits)

        # Create the toolkit-specific control that represents the widget.
        self.control = self._create_control(parent)

        # The layers in the panel.
        #
        # { str name : wx.Window layer }
        self._layers = {}
        self._headers = {}

        return

    # ------------------------------------------------------------------------
    # 'Expandale' interface.
    # ------------------------------------------------------------------------

    def add_panel(self, name, layer):
        """ Adds a layer with the specified name.

        All layers are hidden when they are added.  Use 'show_layer' to make a
        layer visible.

        """

        parent = self.control
        sizer = self.control.GetSizer()

        # Add the heading text.
        header = self._create_header(parent, text=name)
        sizer.Add(header, 0, wx.EXPAND)

        # Add the layer to our sizer.
        sizer.Add(layer, 1, wx.EXPAND)

        # All layers are hidden when they are added.  Use 'show_layer' to make
        # a layer visible.
        sizer.Show(layer, False)

        # fixme: Should we warn if a layer is being overridden?
        self._layers[name] = layer

        return layer

    def remove_panel(self, name):
        """ Removes a layer and its header from the container."""

        if name not in self._layers:
            return

        sizer = self.control.GetSizer()
        panel = self._layers[name]
        header = self._headers[name]
        # sizer.Remove(panel)
        panel.Destroy()
        # sizer.Remove(header)
        header.Destroy()

        sizer.Layout()

        return

    # ------------------------------------------------------------------------
    # Private interface.
    # ------------------------------------------------------------------------

    def _create_control(self, parent):
        """ Create the toolkit-specific control that represents the widget. """

        panel = wx.Panel(parent, -1, style=self.STYLE)
        sizer = wx.BoxSizer(wx.VERTICAL)
        panel.SetSizer(sizer)
        panel.SetAutoLayout(True)

        return panel

    def _create_header(self, parent, text):
        """ Creates a panel header. """

        sizer = wx.BoxSizer(wx.HORIZONTAL)
        panel = wx.Panel(parent, -1, style=wx.CLIP_CHILDREN)
        panel.SetSizer(sizer)
        panel.SetAutoLayout(True)

        # Add the panel header.
        heading = ExpandableHeader(panel, self, title=text)
        sizer.Add(heading.control, 1, wx.EXPAND)

        heading.on_trait_change(self._on_button, "panel_expanded")

        # Resize the panel to match the sizer's minimum size.
        sizer.Fit(panel)

        # hang onto it for when we destroy
        self._headers[text] = panel

        return panel

    # wx event handlers ----------------------------------------------------

    def _on_button(self, event):
        """ called when one of the expand/contract buttons is pressed. """

        header = event
        name = header.title
        visible = header.state

        sizer = self.control.GetSizer()
        sizer.Show(self._layers[name], visible)
        sizer.Layout()

        # fixme: Errrr, maybe we can NOT do this!
        w, h = self.control.GetSize().Get()
        self.control.SetSize((w + 1, h + 1))
        self.control.SetSize((w, h))



# (C) Copyright 2005-2020 Enthought, Inc., Austin, TX
# All rights reserved.
#
# This software is provided without warranty under the terms of the BSD
# license included in LICENSE.txt and may be redistributed only under
# the conditions described in the aforementioned license. The license
# is also available online at http://www.enthought.com/licenses/BSD.txt
#
# Thanks for using Enthought open source!


import unittest

from traits.api import Any, HasTraits, Str

from ..splash_screen_log_handler import SplashScreenLogHandler


class DummySplashScreen(HasTraits):
    text = Str(u"original")


class DummyRecord(object):
    def __init__(self, message):
        self.message = message

    def getMessage(self):
        return self.message


class TestSplashScreenLogHandler(unittest.TestCase):
    def setUp(self):
        self.ss = DummySplashScreen()
        self.sslh = SplashScreenLogHandler(self.ss)

    def test_unicode_message(self):
        self.assertEqual(self.ss.text, u"original")
        message = u"G\u00f6khan"
        self.sslh.emit(DummyRecord(message))
        self.assertEqual(self.ss.text, message + u"...")

    def test_ascii_message(self):
        message = "Goekhan"
        self.sslh.emit(DummyRecord(message))
        self.assertEqual(self.ss.text, message + u"...")



# (C) Copyright 2005-2020 Enthought, Inc., Austin, TX
# All rights reserved.
#
# This software is provided without warranty under the terms of the BSD
# license included in LICENSE.txt and may be redistributed only under
# the conditions described in the aforementioned license. The license
# is also available online at http://www.enthought.com/licenses/BSD.txt
#
# Thanks for using Enthought open source!
""" The 'View' menu """


import logging


from pyface.action.api import Group, MenuManager
from traits.api import Any, Bool, Instance, List, Str
from traits.api import on_trait_change


from .perspective_menu_manager import PerspectiveMenuManager
from .show_view_action import ShowViewAction
from .toggle_view_visibility_action import ToggleViewVisibilityAction


# Logging.
logger = logging.getLogger(__name__)


class ViewMenuManager(MenuManager):
    """ The 'View' menu.

    By default, this menu is displayed on the main menu bar.

    """

    # 'ActionManager' interface --------------------------------------------

    # All of the groups in the manager.
    groups = List(Group)

    # The manager's unique identifier (if it has one).
    id = Str("View")

    # 'MenuManager' interface ---------------------------------------------#

    # The menu manager's name (if the manager is a sub-menu, this is what its
    # label will be).
    name = Str("&View")

    # 'ViewMenuManager' interface -----------------------------------------#

    # Should the perspective menu be shown?
    show_perspective_menu = Bool(True)

    # The workbench window that the menu is part of.
    window = Instance("pyface.workbench.api.WorkbenchWindow")

    # 'Private' interface -------------------------------------------------#

    # The group containing the view hide/show actions.
    _view_group = Any()

    # ------------------------------------------------------------------------
    # 'ActionManager' interface.
    # ------------------------------------------------------------------------

    def _groups_default(self):
        """ Trait initializer. """

        groups = []

        # Add a group containing the perspective menu (if requested).
        if self.show_perspective_menu and len(self.window.perspectives) > 0:
            groups.append(Group(PerspectiveMenuManager(window=self.window)))

        # Add a group containing a 'toggler' for all visible views.
        self._view_group = self._create_view_group(self.window)
        groups.append(self._view_group)

        # Add a group containing an 'Other...' item that will launch a dialog
        # to allow the user to choose a view to show.
        groups.append(self._create_other_group(self.window))

        return groups

    # ------------------------------------------------------------------------
    # 'ViewMenuManager' interface.
    # ------------------------------------------------------------------------

    @on_trait_change(
        "window.active_perspective,window.active_part,"
        "window.views,window.views_items"
    )
    def refresh(self):
        """ Refreshes the checked state of the actions in the menu. """

        logger.debug("refreshing view menu")

        if self._view_group is not None:
            self._clear_group(self._view_group)
            self._initialize_view_group(self.window, self._view_group)
            self.changed = True

        return

    # ------------------------------------------------------------------------
    # Private interface.
    # ------------------------------------------------------------------------

    def _clear_group(self, group):
        """ Remove all items in a group. """

        # fixme: Fix this API in Pyface so there is only one call!
        group.destroy()
        group.clear()

    def _create_other_group(self, window):
        """ Creates a group containing the 'Other...' action. """

        group = Group()
        group.append(ShowViewAction(name="Other...", window=window))

        return group

    def _create_view_group(self, window):
        """ Creates a group containing the view 'togglers'. """

        group = Group()
        self._initialize_view_group(window, group)

        return group

    def _initialize_view_group(self, window, group):
        """ Initializes a group containing the view 'togglers'. """

        views = window.views[:]
        views.sort(key=lambda view: view.name)

        for view in views:
            # fixme: It seems a little smelly to be reaching in to the window
            # layout here. Should the 'contains_view' method be part of the
            # window interface?
            if window.layout.contains_view(view):
                group.append(
                    ToggleViewVisibilityAction(view=view, window=window)
                )

        return



# (C) Copyright 2005-2020 Enthought, Inc., Austin, TX
# All rights reserved.
#
# This software is provided without warranty under the terms of the BSD
# license included in LICENSE.txt and may be redistributed only under
# the conditions described in the aforementioned license. The license
# is also available online at http://www.enthought.com/licenses/BSD.txt
#
# Thanks for using Enthought open source!

""" The text field interface. """


from traits.api import Any, HasTraits, Instance, Str

from pyface.i_widget import IWidget


class IField(IWidget):
    """ The field interface.

    A field is a widget that displays a value and (potentially) allows a user
    to interact with it.
    """

    #: The value held by the field.
    value = Any()

    #: A tooltip for the field.
    tooltip = Str()

    #: An optional context menu for the field.
    context_menu = Instance("pyface.action.menu_manager.MenuManager")

    def show_context_menu(self, x, y):
        """ Create and show the context menu at a position. """

    def _initialize_control(self, control):
        """ Perform any toolkit-specific initialization for the control. """


class MField(HasTraits):
    """ The field mix-in. """

    #: The value held by the field.
    value = Any()

    #: A tooltip for the field.
    tooltip = Str()

    #: An optional context menu for the field.
    context_menu = Instance("pyface.action.menu_manager.MenuManager")

    # ------------------------------------------------------------------------
    # IWidget interface
    # ------------------------------------------------------------------------

    def _add_event_listeners(self):
        """ Set up toolkit-specific bindings for events """
        super(MField, self)._add_event_listeners()
        self.on_trait_change(self._value_updated, "value", dispatch="ui")
        self.on_trait_change(self._tooltip_updated, "tooltip", dispatch="ui")
        self.on_trait_change(
            self._context_menu_updated, "context_menu", dispatch="ui"
        )
        if self.control is not None and self.context_menu is not None:
            self._observe_control_context_menu()

    def _remove_event_listeners(self):
        """ Remove toolkit-specific bindings for events """
        if self.control is not None and self.context_menu is not None:
            self._observe_control_context_menu(remove=True)
        self.on_trait_change(
            self._value_updated, "value", dispatch="ui", remove=True
        )
        self.on_trait_change(
            self._tooltip_updated, "tooltip", dispatch="ui", remove=True
        )
        self.on_trait_change(
            self._context_menu_updated,
            "context_menu",
            dispatch="ui",
            remove=True,
        )
        super(MField, self)._remove_event_listeners()

    # ------------------------------------------------------------------------
    # Private interface
    # ------------------------------------------------------------------------

    def _create(self):
        """ Creates the toolkit specific control.

        This method should create the control and assign it to the
        :py:attr:``control`` trait.
        """
        self.control = self._create_control(self.parent)
        self._initialize_control()
        self._add_event_listeners()

        self.show(self.visible)
        self.enable(self.enabled)

    def _initialize_control(self):
        """ Perform any toolkit-specific initialization for the control. """
        self._set_control_tooltip(self.tooltip)

    def _update_value(self, value):
        """ Handle a change to the value from user interaction

        This is a method suitable for calling from a toolkit event handler.
        """
        self.value = self._get_control_value()

    def _get_control(self):
        """ If control is not passed directly, get it from the trait. """
        control = self.control
        if control is None:
            raise RuntimeError("Toolkit control does not exist.")
        return control

    # Toolkit control interface ---------------------------------------------

    def _get_control_value(self):
        """ Toolkit specific method to get the control's value. """
        raise NotImplementedError

    def _set_control_value(self, value):
        """ Toolkit specific method to set the control's value. """
        raise NotImplementedError

    def _observe_control_value(self, remove=False):
        """ Toolkit specific method to change the control value observer. """
        raise NotImplementedError

    def _get_control_tooltip(self):
        """ Toolkit specific method to get the control's tooltip. """
        raise NotImplementedError

    def _set_control_tooltip(self, tooltip):
        """ Toolkit specific method to set the control's tooltip. """
        raise NotImplementedError

    def _observe_control_context_menu(self, remove=False):
        """ Toolkit specific method to change the control menu observer.

        This should use _handle_control_context_menu as the event handler.
        """
        raise NotImplementedError

    def _handle_control_context_menu(self, event):
        """ Handle a context menu event.

        This should call show_context_menu with appropriate position x and y
        arguments.

        The function signature will likely vary from toolkit to toolkit.
        """
        raise NotImplementedError

    # Trait change handlers -------------------------------------------------

    def _value_updated(self, value):
        if self.control is not None:
            self._set_control_value(value)

    def _tooltip_updated(self, tooltip):
        if self.control is not None:
            self._set_control_tooltip(tooltip)

    def _context_menu_updated(self, old, new):
        if self.control is not None:
            if new is None:
                self._observe_control_context_menu(remove=True)
            if old is None:
                self._observe_control_context_menu()



# (C) Copyright 2005-2020 Enthought, Inc., Austin, TX
# All rights reserved.
#
# This software is provided without warranty under the terms of the BSD
# license included in LICENSE.txt and may be redistributed only under
# the conditions described in the aforementioned license. The license
# is also available online at http://www.enthought.com/licenses/BSD.txt
#
# Thanks for using Enthought open source!

""" Abstract base class for table content providers. """


from .content_provider import ContentProvider


class TableContentProvider(ContentProvider):
    """ Abstract base class for table content providers.

    Table content providers are used by (surprise, surprise) table viewers!

    """

    # This class currently does not specialize the base class in any way.
    # It is here to (hopefully) make the APIs for the viewer, content and label
    # provider classes more consistent.  In particular, if you are building a
    # table viewer you sub-class 'TableContentProvider' and
    # 'TableLabelProvider'.  For a tree viewer you sub-class
    # 'TreeContentProvider' and 'TreeLabelProvider'  instead of some
    # combination of the specific and generic viewer classes as in JFace.
    pass



# (C) Copyright 2005-2020 Enthought, Inc., Austin, TX
# All rights reserved.
#
# This software is provided without warranty under the terms of the BSD
# license included in LICENSE.txt and may be redistributed only under
# the conditions described in the aforementioned license. The license
# is also available online at http://www.enthought.com/licenses/BSD.txt
#
# Thanks for using Enthought open source!

from traits.api import Bool, Enum, HasTraits, Str, Tuple


from pyface.tasks.i_task_pane import ITaskPane


class IDockPane(ITaskPane):
    """ A pane that is useful but unessential for a task.

    Dock panes are arranged around the central pane in dock areas, and can, in
    general, be moved, resized, and hidden by the user.
    """

    # If enabled, the pane will have a button to close it, and a visibility
    # toggle button will be added to the View menu. Otherwise, the pane's
    # visibility will only be adjustable programmatically, though the 'visible'
    # attribute.
    closable = Bool(True)

    # The dock area in which the pane is currently present.
    dock_area = Enum("left", "right", "top", "bottom")

    # Whether the pane can be detached from the main window.
    floatable = Bool(True)

    # Whether the pane is currently detached from the main window.
    floating = Bool(False)

    # Whether the pane can be moved from one dock area to another.
    movable = Bool(True)

    # The size of the dock pane. Note that this value is read-only.
    size = Tuple()

    # Whether the pane is currently visible.
    visible = Bool(False)

    # ------------------------------------------------------------------------
    # 'IDockPane' interface.
    # ------------------------------------------------------------------------

    def create_contents(self, parent):
        """ Create and return the toolkit-specific contents of the dock pane.
        """

    def hide(self):
        """ Convenience method to hide the dock pane.
        """

    def show(self):
        """ Convenience method to show the dock pane.
        """


class MDockPane(HasTraits):
    """ Mixin containing common code for toolkit-specific implementations.
    """

    # 'IDockPane' interface ------------------------------------------------

    closable = Bool(True)
    dock_area = Enum("left", "right", "top", "bottom")
    floatable = Bool(True)
    floating = Bool(False)
    movable = Bool(True)
    size = Tuple()
    visible = Bool(False)
    caption_visible = Bool(True)
    dock_layer = Bool(0)

    # ------------------------------------------------------------------------
    # 'IDockPane' interface.
    # ------------------------------------------------------------------------

    def hide(self):
        """ Convenience method to hide the dock pane.
        """
        self.visible = False

    def show(self):
        """ Convenience method to show the dock pane.
        """
        self.visible = True



# (C) Copyright 2005-2020 Enthought, Inc., Austin, TX
# All rights reserved.
#
# This software is provided without warranty under the terms of the BSD
# license included in LICENSE.txt and may be redistributed only under
# the conditions described in the aforementioned license. The license
# is also available online at http://www.enthought.com/licenses/BSD.txt
#
# Thanks for using Enthought open source!
""" An adapter model that inverts all of its row/column targets. Use
this class with the CompositeGridModel to make models with different
orientations match, or use it to visually flip the data without modifying
the underlying model's sense of row and column. """


from traits.api import Instance


from .grid_model import GridModel


class InvertedGridModel(GridModel):
    """ An adapter model that inverts all of its row/column targets. Use
    this class with the CompositeGridModel to make models with different
    orientations match, or use it to visually flip the data without modifying
    the underlying model's sense of row and column. """

    model = Instance(GridModel, ())

    # ------------------------------------------------------------------------
    # 'GridModel' interface.
    # ------------------------------------------------------------------------

    def get_column_count(self):

        return self.model.get_row_count()

    def get_column_name(self, index):

        return self.model.get_row_name(index)

    def get_cols_drag_value(self, cols):

        return self.model.get_rows_drag_value(cols)

    def get_cols_selection_value(self, cols):

        return self.model.get_rows_selection_value(cols)

    def get_column_context_menu(self, col):

        return self.model.get_row_context_menu(col)

    def sort_by_column(self, col, reverse=False):

        return self.model.sort_by_row(col, reverse)

    def is_column_read_only(self, index):

        return self.model.is_row_read_only(index)

    def get_row_count(self):

        return self.model.get_column_count()

    def get_row_name(self, index):

        return self.model.get_column_name(index)

    def get_rows_drag_value(self, rows):

        return self.model.get_cols_drag_value(rows)

    def get_rows_selection_value(self, rows):

        return self.model.get_cols_selection_value(rows)

    def get_row_context_menu(self, row):

        return self.model.get_col_context_menu(row)

    def sort_by_row(self, row, reverse=False):

        return self.model.sort_by_col(row, reverse)

    def is_row_read_only(self, index):

        return self.model.is_column_read_only(index)

    def delete_rows(self, pos, num_rows):

        return self.model.delete_cols(pos, num_rows)

    def insert_cols(self, pos, num_rows):

        return self.model.insert_rows(pos, num_rows)

    def get_value(self, row, col):

        return self.model.get_value(col, row)

    def get_cell_drag_value(self, row, col):

        return self.model.get_cell_drag_value(col, row)

    def get_cell_selection_value(self, row, col):

        return self.model.get_cell_selection_value(col, row)

    def resolve_selection(self, selection_list):

        return self.model.resolve_selection(selection_list)

    def get_cell_context_menu(self, row, col):

        return self.model.get_cell_context_menu(col, row)

    def set_value(self, row, col, value):

        return self.model.set_value(col, row, value)

    def is_cell_empty(self, row, col):

        return self.model.is_cell_empty(col, row)

    def is_cell_editable(self, row, col):

        return self.model.is_cell_editable(col, row)



# (C) Copyright 2005-2020 Enthought, Inc., Austin, TX
# All rights reserved.
#
# This software is provided without warranty under the terms of the BSD
# license included in LICENSE.txt and may be redistributed only under
# the conditions described in the aforementioned license. The license
# is also available online at http://www.enthought.com/licenses/BSD.txt
#
# Thanks for using Enthought open source!
""" An action that toggles a view's visibility (ie. hides/shows it). """


from pyface.workbench.api import IView
from traits.api import Delegate, Instance


from .workbench_action import WorkbenchAction


class ToggleViewVisibilityAction(WorkbenchAction):
    """ An action that toggles a view's visibility (ie. hides/shows it). """

    # 'Action' interface ---------------------------------------------------

    # The action's unique identifier (may be None).
    id = Delegate("view", modify=True)

    # The action's name (displayed on menus/tool bar tools etc).
    name = Delegate("view", modify=True)

    # The action's style.
    style = "toggle"

    # 'ViewAction' interface -----------------------------------------------

    # The view that we toggle the visibility for.
    view = Instance(IView)

    # ------------------------------------------------------------------------
    # 'Action' interface.
    # ------------------------------------------------------------------------

    def destroy(self):
        """ Called when the action is no longer required. """

        if self.view is not None:
            self._remove_view_listeners(self.view)

    def perform(self, event):
        """ Perform the action. """

        self._toggle_view_visibility(self.view)

        return

    # ------------------------------------------------------------------------
    # Private interface.
    # ------------------------------------------------------------------------

    # Trait change handlers ------------------------------------------------

    def _view_changed(self, old, new):
        """ Static trait change handler. """

        if old is not None:
            self._remove_view_listeners(old)

        if new is not None:
            self._add_view_listeners(new)

        self._refresh_checked()

        return

    # Methods -------------------------------------------------------------#

    def _add_view_listeners(self, view):
        """ Add listeners for trait events on a view. """

        view.on_trait_change(self._refresh_checked, "visible")
        view.on_trait_change(self._refresh_checked, "window")

    def _remove_view_listeners(self, view):
        """ Add listeners for trait events on a view. """

        view.on_trait_change(self._refresh_checked, "visible", remove=True)
        view.on_trait_change(self._refresh_checked, "window", remove=True)

    def _refresh_checked(self):
        """ Refresh the checked state of the action. """

        self.checked = (
            self.view is not None
            and self.view.window is not None
            and self.view.visible
        )

    def _toggle_view_visibility(self, view):
        """ Toggle the visibility of a view. """

        if view.visible:
            view.hide()

        else:
            view.show()

        return



# (C) Copyright 2007 Riverbank Computing Limited
# (C) Copyright 2005-2020 Enthought, Inc., Austin, TX
# All rights reserved.
#
# This software is provided without warranty under the terms of the BSD
# license included in LICENSE.txt and may be redistributed only under
# the conditions described in the aforementioned license. The license
# is also available online at http://www.enthought.com/licenses/BSD.txt
#
# Thanks for using Enthought open source!


import sys

from traits.trait_notifiers import set_ui_handler, ui_handler

from pyface.qt import QtCore, QtGui, qt_api
from pyface.base_toolkit import Toolkit
from .gui import GUI

if qt_api == "pyqt":
    # Check the version numbers are late enough.
    if QtCore.QT_VERSION < 0x040200:
        raise RuntimeError(
            "Need Qt v4.2 or higher, but got v%s" % QtCore.QT_VERSION_STR
        )

    if QtCore.PYQT_VERSION < 0x040100:
        raise RuntimeError(
            "Need PyQt v4.1 or higher, but got v%s" % QtCore.PYQT_VERSION_STR
        )

# It's possible that it has already been initialised.
_app = QtGui.QApplication.instance()

if _app is None:
    _app = QtGui.QApplication(sys.argv)


# create the toolkit object
toolkit_object = Toolkit("pyface", "qt4", "pyface.ui.qt4")


# ensure that Traits has a UI handler appropriate for the toolkit.
if ui_handler is None:
    # Tell the traits notification handlers to use this UI handler
    set_ui_handler(GUI.invoke_later)



# (C) Copyright 2005-2020 Enthought, Inc., Austin, TX
# All rights reserved.
#
# This software is provided without warranty under the terms of the BSD
# license included in LICENSE.txt and may be redistributed only under
# the conditions described in the aforementioned license. The license
# is also available online at http://www.enthought.com/licenses/BSD.txt
#
# Thanks for using Enthought open source!


import unittest

from traits.testing.unittest_tools import UnittestTools

from pyface.gui_application import GUIApplication
from ..gui_application_action import GUIApplicationAction
from ..action_event import ActionEvent


class TestAction(unittest.TestCase, UnittestTools):
    def setUp(self):
        self.application = GUIApplication()

    def test_defaults(self):
        action = GUIApplicationAction()
        event = ActionEvent()

        # does nothing, but shouldn't error
        action.perform(event)

        self.assertTrue(action.enabled)
        self.assertTrue(action.visible)
        self.assertIsNone(action.object)

    def test_application(self):
        action = GUIApplicationAction(application=self.application)
        event = ActionEvent()

        # does nothing, but shouldn't error
        action.perform(event)

        self.assertTrue(action.enabled)
        self.assertTrue(action.visible)
        self.assertEqual(action.object, self.application)

    def test_application_changed(self):
        action = GUIApplicationAction()

        self.assertIsNone(action.object)

        with self.assertTraitChanges(action, "object", 1):
            action.application = self.application

        self.assertEqual(action.object, self.application)

        with self.assertTraitChanges(action, "object", 1):
            action.application = None

        self.assertIsNone(action.object)

    def test_destroy(self):
        action = GUIApplicationAction(application=self.application)

        action.destroy()

        self.assertEqual(action.object, None)



# (C) Copyright 2005-2020 Enthought, Inc., Austin, TX
# All rights reserved.
#
# This software is provided without warranty under the terms of the BSD
# license included in LICENSE.txt and may be redistributed only under
# the conditions described in the aforementioned license. The license
# is also available online at http://www.enthought.com/licenses/BSD.txt
#
# Thanks for using Enthought open source!

""" A Layered panel. """


import wx
from wx.lib.scrolledpanel import ScrolledPanel


from traits.api import Any, Str, Int


from .widget import Widget


class LayeredPanel(Widget):
    """ A Layered panel.

    A layered panel contains one or more named layers, with only one layer
    visible at any one time (think of a 'tab' control minus the tabs!).  Each
    layer is a toolkit-specific control.

    """

    # The default style.
    STYLE = wx.CLIP_CHILDREN

    # "Layered Panel' interface --------------------------------------------

    # The toolkit-specific control of the currently displayed layer.
    current_layer = Any()

    # The name of the currently displayed layer.
    current_layer_name = Str()

    # The minimum for the panel, which is the maximum of the minimum
    # sizes of the layers
    min_width = Int(0)
    min_height = Int(0)

    # ------------------------------------------------------------------------
    # 'object' interface.
    # ------------------------------------------------------------------------

    def __init__(self, parent, **traits):
        """ Creates a new LayeredPanel. """

        # Base class constructor.
        super(LayeredPanel, self).__init__(**traits)

        # Create the toolkit-specific control that represents the widget.
        self.control = self._create_control(parent)

        # The layers in the panel.
        #
        # { str name : wx.Window layer }
        self._layers = {}

        return

    # ------------------------------------------------------------------------
    # 'LayeredPanel' interface.
    # ------------------------------------------------------------------------

    def add_layer(self, name, layer):
        """ Adds a layer with the specified name.

        All layers are hidden when they are added.  Use 'show_layer' to make a
        layer visible.

        """

        # Add the layer to our sizer.
        sizer = self.control.GetSizer()
        sizer.Add(layer, 1, wx.EXPAND)

        # All layers are hidden when they are added.  Use 'show_layer' to make
        # a layer visible.
        sizer.Show(layer, False)

        # fixme: Should we warn if a layer is being overridden?
        self._layers[name] = layer

        # fixme: The minimum size stuff that was added for linux broke the
        # sizing on Windows (at least for the preference dialog).  The
        # preference dialog now sets the minimum width and height to -1 so
        # that this layout code doesn't get executed.
        if self.min_width != -1 or self.min_height != -1:
            if layer.GetSizer() is None:
                return layer

            min_size = layer.GetSizer().CalcMin()
            needs_layout = False
            if min_size.GetWidth() > self.min_width:
                self.min_width = min_size.GetWidth()
                needs_layout = True
            if min_size.GetHeight() > self.min_height:
                self.min_height = min_size.GetHeight()
                needs_layout = True

            if needs_layout:
                # Reset our size hints and relayout
                self.control.SetSizeHints(self.min_width, self.min_height)
                self.control.GetSizer().Layout()

                # fixme: Force our parent to reset it's size hints to its
                # minimum
                parent = self.control.GetParent()
                parent.GetSizer().SetSizeHints(parent)
                parent.GetSizer().Layout()

        return layer

    def show_layer(self, name):
        """ Shows the layer with the specified name. """

        # Hide the current layer (if one is displayed).
        if self.current_layer is not None:
            self._hide_layer(self.current_layer)

        # Show the specified layer.
        layer = self._show_layer(name, self._layers[name])

        return layer

    def has_layer(self, name):
        """ Does the panel contain a layer with the specified name? """

        return name in self._layers

    # ------------------------------------------------------------------------
    # Private interface.
    # ------------------------------------------------------------------------

    def _create_control(self, parent):
        """ Create the toolkit-specific control that represents the widget. """

        panel = ScrolledPanel(parent, -1, style=self.STYLE)
        sizer = wx.BoxSizer(wx.VERTICAL)
        panel.SetSizer(sizer)
        panel.SetAutoLayout(True)
        panel.SetupScrolling()

        return panel

    def _hide_layer(self, layer):
        """ Hides the specified layer. """

        sizer = self.control.GetSizer()
        sizer.Show(layer, False)
        sizer.Layout()

    def _show_layer(self, name, layer):
        """ Shows the specified layer. """

        sizer = self.control.GetSizer()
        sizer.Show(layer, True)
        sizer.Layout()

        self.current_layer = layer
        self.current_layer_name = name

        return layer
